﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Diagnostics.Contracts;
using Ennf.Properties;
using MathNet.Numerics;

namespace Ennf.Functions.ActivationFunctions
{
    /// <summary>
    /// Calculates the value of the Logistic Sigmoid Function
    /// 1/1+exp(-a) and it's derivative
    /// </summary>
    public class LogisticSigmoidFunction : IActivationFunction,IParentNetwork
    {
        #region IParentNetwork interface
        private INetwork _parentNetwork;
        public INetwork ParentNetwork
        {
            get
            {
                Contract.Ensures(Contract.Result<INetwork>() == _parentNetwork);
                return _parentNetwork;
            }
        }
        INetwork IParentNetwork.ParentNetwork
        {
            set
            {
                Contract.Ensures(_parentNetwork == value);
                _parentNetwork = value;
            }
        }
        #endregion

        [Pure]
        public string Name
        {
            get { return Resources.LogisticSigmoidFunctionName; }
        }

        /// <summary>
        /// Calculates the value of the logistic sigmoid function at the point inputValue
        /// </summary>
        /// <param name="inputValue">Must be a single value only</param>
        /// <returns>A double containing the value of the logistic sigmoid function at the given point</returns>
        public double Calculate(params double[] inputValue)
        {
            if (inputValue == null || inputValue.Length != 1)
                throw new ArgumentException("The input value of the Logistic Sigmoid Function must be exactly one parameter, not null or more than one");

            return  SpecialFunctions.Logistic(inputValue[0]);
        }

        /// <summary>
        /// Calculates the value of the derivative of the logistic sigmoid function at the point inputValue
        /// </summary>
        /// <param name="inputValue">Must be a single value only</param>
        /// <returns>A double containing the value of the derivative of the logistic sigmoid function at the given point</returns>
        public double CalculateDerivative(params double[] inputValue)
        {
            if (inputValue == null || inputValue.Length != 1)
                throw new ArgumentException("The input value of the derivative of the Logistic Sigmoid Function must be exactly one parameter, not null or more than one");

            double calcualtedValue = Calculate(inputValue);

            return calcualtedValue*(1 - calcualtedValue);
        }
    }
}
