﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using Ennf.Properties;
using MathNet.Numerics.LinearAlgebra.Double;
using MathNet.Numerics.LinearAlgebra.Generic;
using MathNet.Numerics.Statistics;

namespace Ennf.Functions.ConvergenceSpeedFunctions
{
    /// <summary>
    /// Calculates the mean of the distances of errors measured; the same could probably be achieved
    /// by usind the derivative of the Learning rule, but this will do for now
    /// </summary>
    public class MovingAverageConvergenceSpeed : IConvergenceSpeedFunction,IParentNetwork
    {
        #region IParentNetwork interface
        public INetwork ParentNetwork { get; private set; }
        INetwork IParentNetwork.ParentNetwork { set { ParentNetwork = value; } }
        #endregion

        private readonly IList<double> _errors = new List<double>();

        [ContractInvariantMethod]
        private void Invariants()
        {
            Contract.Invariant(_errors != null);
        }

        public int Decay { get; set; }

        public MovingAverageConvergenceSpeed(int decayCount)
        {
            Contract.Ensures(Decay == decayCount);
            Decay = decayCount;
        }

        public MovingAverageConvergenceSpeed()
        {
            Contract.Ensures(Decay == 10);
            Decay = 10;
        }

        [Pure]
        public string Name
        {
            get { return Resources.MovingAverageConvergenceFunctionName; }
        }

        public double Calculate(params double[] inputValue)
        {
            Contract.Requires<ArgumentNullException>(inputValue != null);
            Contract.Requires(inputValue.Length >= 1);

            Vector<double> distances;

            if (inputValue.Length == 1)
            {
                distances = new DenseVector(inputValue.Length);
                distances[0] = inputValue[0];
            }    
            else
            {
                distances = new DenseVector(inputValue.Length-1);
                for (int i = inputValue.Length-1; i > 0; i--)
                {
                    distances[inputValue.Length - (i+1)] = Math.Abs(inputValue[i] - inputValue[i - 1]);
                }
            }
            return distances.Mean();
        }

        public double Calculate()
        {
            Contract.Ensures(1<= _errors.Count);
            return Calculate(_errors.ToArray());
        }

        public void RegisterError(double errorValue)
        {
            Contract.Ensures(_errors.Count <= Decay && _errors.Count > 0);
            _errors.Add(errorValue);
            if (_errors.Count > Decay)
                _errors.RemoveAt(0);
        }
    }
}
