﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Globalization;
using System.Linq;
using Ennf.NetworkModels.BasicFeedForward;
using Ennf.Properties;
using MathNet.Numerics.LinearAlgebra.Double;
using MathNet.Numerics.LinearAlgebra.Generic;

namespace Ennf.Functions.ErrorFunctions
{
    /// <summary>
    /// Calculates the Sum of Squares Error E(w)= 1/2 * SUM(target - output)^2
    /// </summary>
    public class SumOfSquaresFunction : IErrorFunction,IParentNetwork
    {
        #region IParentNetwork interface
        private INetwork _parentNetwork;
        public INetwork ParentNetwork
        {
            get
            {
                Contract.Ensures(Contract.Result<INetwork>() == _parentNetwork);
                return _parentNetwork;
            }
        }
        INetwork IParentNetwork.ParentNetwork 
        { 
            set
            {
                Contract.Ensures(value == _parentNetwork);
                _parentNetwork = value;
            } 
        }
        #endregion
        
        [Pure]
        public string Name
        {
            get { return Resources.SumOfSquaresErrorFunction; }
        }

        public double Calculate(params double[] inputValue)
        {
            IList<double> arrayOutputValues = new List<double>();
            IList<double> arrayTargetValues = new List<double>();
            BasicFeedForwardNetwork feedForwardNetwork = ParentNetwork as BasicFeedForwardNetwork;
            if (feedForwardNetwork == null)
                throw new ArgumentException(Resources.ParentNetworkIsNotBasicFeedForward);
            if (ParentNetwork.Target == null)
                throw new ArgumentException(Resources.TargetFunctionIsNull);

            ParentNetwork.Target.Reset();
            int counter = 0;
            Dictionary<string, object> additionalInformation = new Dictionary<string, object>();
            foreach (BasicFeedForwardNode node in (from BasicFeedForwardOutputConnection outputData in ParentNetwork.OutputConnections select outputData.InputNodeInternal))
            {
                ParentNetwork.Target.MoveNext();
                arrayOutputValues.Add(node.OutputValue);
                additionalInformation.Add(string.Format(CultureInfo.InvariantCulture, "Output Value {0}", counter), node.OutputValue);
                arrayTargetValues.Add(ParentNetwork.Target.Current);
                additionalInformation.Add(string.Format(CultureInfo.InvariantCulture, "Target Value {0}", counter++), ParentNetwork.Target.Current);
            }

            Matrix<double> outputValues = new DenseMatrix(1, arrayOutputValues.Count);
            outputValues.SetRow(0, arrayOutputValues.ToArray());
            Matrix<double> targetValues = new DenseMatrix(1, arrayTargetValues.Count);
            targetValues.SetRow(0, arrayTargetValues.ToArray());

            feedForwardNetwork.Tracer.LogWriter.Write("Parameter", additionalInformation);
            return Calculate(outputValues, targetValues);
        }

        public double Calculate(Matrix<double> inputValues, Matrix<double> targetValues)
        {
            Contract.Requires<ArgumentNullException>(targetValues != null);

            double toreturn = 0.0;

            Matrix<double> tosum = targetValues.Subtract(inputValues);
            var rowEnumer = tosum.RowEnumerator().GetEnumerator();

            while (rowEnumer.MoveNext())
            {
                Vector<double> row = rowEnumer.Current.Item2;
                toreturn += row.Sum(d => d*d);
            }

            return toreturn/2;
        }

        public double CalculateDerivedDistance(double targetValue, double isValue)
        {
            Contract.Ensures(Contract.Result<double>() == (targetValue - isValue));
            return targetValue - isValue;
        }
    }
}
