﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using Ennf.Infrastructure;
using Ennf.NetworkModels.BasicFeedForward;

namespace Ennf.LearningRules.SupervisedLearning.ErrorBackpropagation
{
    /// <summary>
    /// Extension class which does the actual Error calculation and propagation
    /// </summary>
    static class BackpropagationError
    {

        public static double Error(this BasicFeedForwardNode node)
        {
            // See if we already calculated it for this iteration
            if (ErrorValue[node.ParentNetwork].ContainsKey(node))
                return ErrorValue[node.ParentNetwork][node];

            double toreturn = 0;
            var hasOutputConnections = from outputConnection in node.OutputConnections
                                       where outputConnection is BasicFeedForwardOutputConnection
                                       select outputConnection;

            // if we have an output, we have to calculate the Error value
            if (hasOutputConnections.Any())
            {
                toreturn = hasOutputConnections.Cast<BasicFeedForwardOutputConnection>().Sum(connection => connection.Error());
            }
            // if not, we have to get the error from the previous nodes and calculate it
            else
            {
                toreturn = node.OutputConnections.Sum(connection => connection.Weight*connection.OutputNodeInternal.Error());
                toreturn *= node.ActivationFunction.CalculateDerivative(node.SummationValue);
            }

            ErrorValue[node.ParentNetwork][node] = toreturn;

            return toreturn;
        }

        public static double Error(this BasicFeedForwardOutputConnection outputConnection)
        {
            Contract.Requires<ArgumentNullException>(outputConnection != null);
            Contract.Requires(outputConnection.ParentNetwork != null);
            Contract.Requires(outputConnection.ParentNetwork.ErrorFunction != null);
            Contract.Ensures(outputConnection.ParentNetwork.ErrorFunction != null);
            // if we are at the end of the Input, start from the beginning
            if (!outputConnection.ParentNetwork.Target.MoveNext())
            {
                outputConnection.ParentNetwork.Target.Reset();
                if (!outputConnection.ParentNetwork.Target.MoveNext())
                    throw new EnnfApplicationException("Could not reset Iterator");
            }

            // Calculate error based on target
            double toreturn = outputConnection.InputNodeInternal.ActivationFunction.CalculateDerivative(
                outputConnection.InputNodeInternal.SummationValue) *
                              outputConnection.ParentNetwork.ErrorFunction.CalculateDerivedDistance(outputConnection.ParentNetwork.Target.Current,
                                                                                                    outputConnection.OutputValue);
            return toreturn;
        }

        public static void StartIteration(this INetwork network)
        {
            ErrorValue[network] = new Dictionary<INode, double>();
        }

        [ContractInvariantMethod]
        private static void Invariants()
        {
            Contract.Invariant(ErrorValue != null);
        }

        private static readonly Dictionary<INetwork,Dictionary<INode,double>> ErrorValue = new Dictionary<INetwork, Dictionary<INode, double>>(); 
    }
}
