﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Xml.Serialization;
using Ennf.NetworkModels.BasicFeedForward;
using Ennf.Properties;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.LearningRules.SupervisedLearning.ErrorBackpropagation
{
    /// <summary>
    /// Implements the Error Backprop Strategy for FeedForward Networks
    /// </summary>
    [XmlType(Namespace = "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd")]
    [XmlRootAttribute(Namespace = "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd")]
    public class ErrorBackpropagationRule : ISupervisedLearningRule,IParentNetwork
    {
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "value"), XmlAttribute]
        public String Type
        {
            [Pure]
            get { return this.GetType().ToString(); }
            set { }
        }

        #region Tracing
        [Dependency]
        [XmlIgnore]
        public TraceManager TraceManager { get; set; }
        #endregion

        #region IParentNetwork interface
        [XmlIgnore]
        public INetwork ParentNetwork { get; private set; }
        INetwork IParentNetwork.ParentNetwork { set { ParentNetwork = value; } }
        #endregion

        #region ISupervisedLearningRule Implementation
        public string Name
        {
            [Pure]
            get { return Resources.ErrorBackpropFunctionName; }
        }

        public double LearningRate { get; set; }

        public void CalculateNewWeights(INetwork network)
        {
            Contract.Requires<ArgumentNullException>(network != null);
            Contract.Requires(network.Connections != null);
            Contract.Requires<InvalidOperationException>(LearningRate != 0.0 );
            Contract.Ensures(network.Connections.Count >= 0);
            
            ParentNetwork.StartIteration();

            var selectWeights = from connection in network.Connections
                                where
                                    !(connection is BasicFeedForwardInputConnection) &&
                                    !(connection is BasicFeedForwardOutputConnection)
                                select connection;

            foreach (BasicFeedForwardConnection selectWeight in selectWeights)
            {
                double deltaWeight = LearningRate*selectWeight.OutputNodeInternal.Error()*
                                     selectWeight.InputNodeInternal.OutputValue;
                selectWeight.Weight += deltaWeight;
            }
            
        }
        #endregion
    }
}
