﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System.Collections.Generic;
using Ennf.Functions;
using Ennf.LearningRules;

namespace Ennf
{
    /// <summary>
    /// Node Network interface for ennf
    /// </summary>
    public interface INetwork
    {
        #region Ennf Object Model
        /// <summary>
        /// List of associated Nodes belonging to a Network
        /// </summary>
        ICollection<INode> Nodes { get; }

        /// <summary>
        /// List of all Connections (General, Input, Output) between Nodes belonging to this Network
        /// </summary>
        ICollection<IConnection> Connections { get; }

        /// <summary>
        /// List of Input Connections belonging to this Network
        /// </summary>
        ICollection<IInputConnection> InputConnections { get; }

        /// <summary>
        /// List of Output Connections belonging to this Network
        /// </summary>
        ICollection<IOutputConnection> OutputConnections { get; }

        /// <summary>
        /// The parent Network Model to which the Network belongs
        /// </summary>
        INetworkModel Parent { get; }

        /// <summary>
        /// The input adapter
        /// </summary>
        IInput Input { get; set; }

        /// <summary>
        /// The output adapter
        /// </summary>
        IOutput Output { get; set; }

        /// <summary>
        /// Targetvalues needed 
        /// </summary>
        IInput Target { get; set; }
        #endregion

        #region Basic Node Operations

        /// <summary>
        /// Creates a node for the Network Model
        /// </summary>
        /// <returns>A new node</returns>
        INode CreateNode();

        #endregion

        #region Basic Connection Operations
        /// <summary>
        /// Creates a connection for the Network Model
        /// </summary>
        /// <returns>A new general connection</returns>
        IConnection CreateConnection();

        /// <summary>
        /// Creates a input connection for the network model
        /// </summary>
        /// <returns>A new input connection</returns>
        IInputConnection CreateInputConnection();

        /// <summary>
        /// Creates a output connection for the network model
        /// </summary>
        /// <returns>A new output connection</returns>
        IOutputConnection CreateOutputConnection();

        /// <summary>
        /// Initialization Function used to initialize the weight of connections
        /// </summary>
        IInitializationFunction InitializationFunction { get; set; }

        /// <summary>
        /// Learning Rule to use to adjust the weights of connections
        /// </summary>
        ILearningRule LearningRule { get; set; }

        /// <summary>
        /// Error Function to determine if convergence is achieved
        /// </summary>
        IErrorFunction ErrorFunction { get; set; }

        /// <summary>
        /// Function measures the convergence of error
        /// </summary>
        IConvergenceSpeedFunction ConvergenceSpeedFunction { get; set; }
        #endregion

        #region Basic Network Operations

        /// <summary>
        /// Takes the next set of input data and executes the calculation operation
        /// </summary>
        void Calculate();

        /// <summary>
        /// Adjust the weights based on the learning strategy
        /// </summary>
        void Learn();

        #endregion

        #region Network Properties

        /// <summary>
        /// Name of the network as provided by the user
        /// </summary>
        string Name { get; set; }

        /// <summary>
        /// Description of the network as provided by the user
        /// </summary>
        string Description { get; set; }
        #endregion
    }
}