﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.IO;
using System.Runtime.Serialization;
using System.Security.Permissions;
using Ennf.Properties;
using MathNet.Numerics.LinearAlgebra.Double;
using MathNet.Numerics.LinearAlgebra.Double.IO;
using MathNet.Numerics.LinearAlgebra.Generic;

namespace Ennf.Infrastructure
{
    /// <summary>
    /// Class to read a CSV file filled with input data
    /// </summary>
    [Serializable]
    public class CsvStreamInput : IInput
    {
        #region Private Properties
        private Matrix<double> _inputs;
        private IEnumerator<Tuple<int, Vector<double>>> _rowEnumerator;
        private int _rowCounter = -1;
        private bool _disposed;
        #endregion

        #region Private Constants

        private const string DELIMITER = ",";
        private const string INPUTNAME = "Inputs";

        #endregion

        #region Constructors
        /// <summary>
        /// Constructor to read a input stream
        /// </summary>
        /// <param name="inputStream">Stream to read the CSV input from</param>
        public CsvStreamInput(Stream inputStream)
        {
            Contract.Requires<ArgumentNullException>(inputStream != null);
            ReadInput(inputStream);
        }

        /// <summary>
        /// Standard Constructor used by the serialization
        /// </summary>
        public CsvStreamInput()
        {
        }
        #endregion

        #region Serialization
        /// <summary>
        /// Protected constructor used for de-serialization
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        protected CsvStreamInput(SerializationInfo info, StreamingContext context)
        {
            Contract.Requires<ArgumentNullException>(info != null);
            _inputs = info.GetValue(INPUTNAME, typeof (Matrix<double>)) as Matrix<double>;

        }

        /// <summary>
        /// Method to serialize the CsvStreamInput including the input data
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        public virtual void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            Contract.Requires<ArgumentNullException>(info != null);
            info.AddValue(INPUTNAME, _inputs);
        }
        #endregion

        #region IInput Members

        /// <summary>
        /// Gets the current value where the enumerator is standing
        /// </summary>
        public double Current
        {
            get { return GetCurrentValue(); }
        }

        /// <summary>
        /// The IEnumerator Version of Current
        /// </summary>
        object IEnumerator.Current
        {
            get { return GetCurrentValue(); }
        }

        /// <summary>
        /// Central Method which gets the actual value used by the different Current implementations
        /// </summary>
        /// <returns>The current value</returns>
        private double GetCurrentValue()
        {
            Contract.Requires<InvalidOperationException>(_rowEnumerator != null,Resources.EnumeratorNotInitialized);
            Contract.Requires<IndexOutOfRangeException>(_rowCounter <= _rowEnumerator.Current.Item2.Count - 1);

            return _rowEnumerator.Current.Item2[_rowCounter];
        }

        /// <summary>
        /// Moves the Enumerator one position inside of the current DataSet
        /// </summary>
        /// <returns></returns>
        public bool MoveNext()
        {
            if (_rowEnumerator == null)
                ResetInput();
             Contract.Assert(_rowEnumerator != null);   

            return (++_rowCounter <= _rowEnumerator.Current.Item2.Count-1);
        }

        /// <summary>
        /// Moves to the next DataSet
        /// </summary>
        /// <returns></returns>
        public bool MoveNextInput()
        {
            if (_rowEnumerator == null)
                ResetInput();

            // first reset the data counter
            Reset();
            // now move to the next DataSet
            return _rowEnumerator.MoveNext();
        }

        /// <summary>
        /// Resets the enumerator to beginning of the current DataSet
        /// </summary>
        public void Reset()
        {
            _rowCounter = -1;
        }

        /// <summary>
        /// Resets the enumerator to the beginning of the first DataSet
        /// </summary>
        public void ResetInput()
        {
            // Get new Row Enumerator - this points to the first Row of the Matrix
            _rowEnumerator = _inputs.RowEnumerator().GetEnumerator();
            // Set the Counter for access in the array
            Reset();
            if (_rowEnumerator == null)
                throw new EnnfApplicationException("Matrix Enumerator is still null and can't be used");
        }

        #endregion

        /// <summary>
        /// Method to set the input data to the stream
        /// </summary>
        /// <param name="inputStream">Stream to read the CSV input from</param>
        public void ReadInput(Stream inputStream)
        {
            Contract.Requires<ArgumentNullException>(inputStream != null);

            DelimitedReader<DenseMatrix> reader = new DelimitedReader<DenseMatrix>(DELIMITER);
            inputStream.Seek(0, SeekOrigin.Begin);
            _inputs = reader.ReadMatrix(inputStream);
        }

        #region Dispose Pattern
        /// <summary>
        /// Used in case there are unmanaged resources which need to be disposed of
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Internal method according to the Dispose Pattern
        /// </summary>
        /// <remarks>Although nothing needs to be disposed, implemented according to the pattern</remarks>
        /// <param name="disposing"></param>
        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                }
            }
            _disposed = true;
        }
        #endregion

        
    }
}