﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics.Contracts;
using Ennf.Properties;
using Ennf.Utilities;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.NetworkModels.BasicFeedForward
{
    /// <summary>
    /// The basic, weighted connection between two nodes. Unlike InputConnection and OutputConnection,
    /// the weights aren't automatically set to 1, but initialized by the InitializationFunction defined in 
    /// the network.
    /// </summary>
    public partial class BasicFeedForwardConnection : IConnection, IParentNetwork
    {
        #region Tracing and Logging

        [Dependency]
        public LogWriter Logger { [Pure]get; set; }

        [Dependency]
        public TraceManager TraceManager { [Pure]get; set; }

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(Logger != null);
            Contract.Invariant(TraceManager != null);
        }

        #endregion

        #region Properties
        /// <summary>
        /// The network the Connection belongs to
        /// </summary>
        private BasicFeedForwardNetwork _parentNetwork;

        public INetwork ParentNetwork
        {
            [Pure]
            get
            {
                return _parentNetwork;
            }
            set
            {
                Contract.Ensures(_parentNetwork != null, Resources.NotBasicFeedForwardNetwork);
                _parentNetwork = value as BasicFeedForwardNetwork;
                
            }
        }

        /// <summary>
        /// The last value the connection passed on to the output node
        /// </summary>
        public double OutputValue { [Pure]get; protected set; }

        #endregion

        #region Constructor 

        public BasicFeedForwardConnection()
        {
            Logger = new NullLogger();
            TraceManager = new TraceManager(Logger);
        }

        public BasicFeedForwardConnection(BasicFeedForwardNetwork network):this()
        {
            Contract.Requires<ArgumentNullException>(network != null);
            Contract.Requires(network.InitializationFunction != null);
            Contract.Ensures(ParentNetwork != null);
            Contract.Ensures(ParentNetwork.InitializationFunction != null);
            
            // Init the weight according to the initialization function
            Weight = network.InitializationFunction.Calculate();
            ParentNetwork = network;
        }

        #endregion

        /// <summary>
        /// Does the calculation of the weight Inputvalue * Weight
        /// </summary>
        /// <returns>The output value thus calculated</returns>
        /// <seealso cref="OutputValue"/>
        public virtual double Calculate()
        {
            using (TraceManager.StartTrace("Ennf.NetworkModels.BasicFeedForward.Calculation"))
            {
                Dictionary<string, object> additionalInformation = new Dictionary<string, object>
                                                                       {
                                                                           {"ConnectionId", ConnectionId},
                                                                           {"Weight", Weight}
                                                                       };

                // Do the calculation
                OutputValue = Weight*InputNodeInternal.Calculate();

                // Trace the information
                additionalInformation.Add("Return from Connection",OutputValue);
                TraceManager.LogWriter.Write("Parameters","Ennf.NetworkModels.BasicFeedForward.Calculation", additionalInformation);
                return (OutputValue);
            }
        }

        #region IConnection Members

        public virtual INode OutputNode
        {
            [Pure]
            get { return OutputNodeInternal; }
            set
            {
                Contract.Requires<ArgumentException>(value is BasicFeedForwardNode,Resources.OutputNodeWrongType);
                
                OutputNodeInternal = value as BasicFeedForwardNode;
                if (OutputNodeInternal.InputConnections == null)
                    OutputNodeInternal.InputConnections = new Collection<BasicFeedForwardConnection>();
                if (OutputNodeInternal.OutputConnections == null)
                    OutputNodeInternal.OutputConnections = new Collection<BasicFeedForwardConnection>();
                Output_NodeId_FK = OutputNodeInternal.NodeId;

                OutputNodeInternal.InputConnections.Add(this);
            }
        }

        public virtual INode InputNode
        {
            [Pure]
            get { return InputNodeInternal; }
            set
            {
                Contract.Requires<ArgumentException>(value is BasicFeedForwardNode, Resources.InputNodeWrongType);
                
                InputNodeInternal = value as BasicFeedForwardNode;
                if (InputNodeInternal.InputConnections == null)
                    InputNodeInternal.InputConnections = new Collection<BasicFeedForwardConnection>();
                if (InputNodeInternal.OutputConnections == null)
                    InputNodeInternal.OutputConnections = new Collection<BasicFeedForwardConnection>();
                Input_NodeId_FK = InputNodeInternal.NodeId;

                InputNodeInternal.OutputConnections.Add(this);
            }
        }

        #endregion
    }
}