﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using Ennf.Infrastructure;

namespace Ennf.NetworkModels.BasicFeedForward
{
    /// <summary>
    /// Connects the Nodes with the Input; the weight property inherited from the Connection can be set to 
    /// whatever it likes, as this is not used to pass the value to a node
    /// </summary>
    public partial class BasicFeedForwardInputConnection : IInputConnection
    {
        /// <summary>
        /// Counter for how often input data was called from the network for diagnostic reasons
        /// </summary>
        private static int _inputCalled = 1;

        #region Constructors

        public BasicFeedForwardInputConnection()
        {
        }

        public BasicFeedForwardInputConnection(BasicFeedForwardNetwork network)
            : base(network)
        {
            Contract.Requires<ArgumentNullException>(network != null);
            Contract.Requires(network.InitializationFunction != null);
        }

        #endregion

        /// <summary>
        /// While the Input Connection is a IConnection, the Property is not supported for the 
        /// InputConnection
        /// </summary>
        /// <exception cref="InvalidOperationException">This is always null for a input connection and should not be attempted</exception>
        public override INode InputNode
        {
            get { return null; }
            set { throw new InvalidOperationException("InputConnection does not support the InputNode Property"); }
        }

        /// <summary>
        /// Gets the next value from the input stream and passed this to the output node. It does
        /// not use the weight to do this
        /// </summary>
        /// <returns>The value from the input stream</returns>
        public override double Calculate()
        {
            using (TraceManager.StartTrace("Ennf.NetworkModles.BasicFeedForward.Calculation"))
            {
                // if we are at the end of the Input, start from the beginning
                if (!Input.MoveNext())
                {
                    Input.Reset();
                    if (!Input.MoveNext())
                        throw new EnnfApplicationException("Could not reset Iterator");
                }
               
                Dictionary<string, object> additionalInformation = new Dictionary<string, object>
                                                                        {{"Input Connection Id", ConnectionId}};
                    
                // Get Input Value
                double toreturn = Input.Current;
                // Trace Value
                additionalInformation.Add("Return from InputConnection",toreturn);
                additionalInformation.Add("InputCalled",_inputCalled++);
                TraceManager.LogWriter.Write("Parameters", "Ennf.NetworkModels.BasicFeedForward.Calculation", additionalInformation);
                // Return it
                return toreturn;
            }
        }

        #region IInputConnection Members

        public IInput Input
        {
            get { return ParentNetwork.Input; }
        }

        #endregion
    }
}