﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Diagnostics.Contracts;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Formatters.Binary;
using Ennf.DbEntityAdapter;
using Ennf.Functions;
using Ennf.Functions.InitializationFunctions;
using Ennf.Infrastructure;
using Ennf.LearningRules;
using Ennf.Properties;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.NetworkModels.BasicFeedForward
{
    /// <summary>
    /// The network implementation for a strait forward Feed Forward Network
    /// </summary>
    public partial class BasicFeedForwardNetwork : INetwork
    {

        #region Model
        private BasicFeedForwardModel _parent;
        /// <summary>
        /// Reference to the Network Model used
        /// </summary>
        /// <exception cref="ArgumentException">Thrown if the internal setter does not pass a BasicFeedForwardModel</exception>
        public INetworkModel Parent
        {
            get { return _parent; }
            internal set
            {
                using (Tracer.StartTrace("Ennf.NetworksModels"))
                {
                    _parent = value as BasicFeedForwardModel;
                    if (_parent == null)
                        throw new ArgumentException("BasicFeedForwardNetwork expects BasicFeedForwardModel as Type");
                }
            }
        }

        #endregion

        #region Tracing and Logging

        [Dependency]
        public TraceManager Tracer { get; set; }

        [Dependency]
        public LogWriter Logger { get; set; }

        [Dependency]
        public IUnityContainer UnityContainer { get; set; }

        [Dependency]
        public IXmlTypeLoader FunctionLoader { get; set; }

        #endregion

        #region Node Operations

        /// <summary>
        /// This is a inbetween store which the user is handed, when requesting the nodes. If
        /// the user changes anything, a callback into network methods are done and so the 
        /// underlying database is kept in-sync
        /// </summary>
        private ObservableCollection<INode> _nodes;

        /// <summary>
        /// List of Nodes belonging to the Network
        /// </summary>
        /// <remarks>This Property loads the nodes from the database on initial loads</remarks>
        public ICollection<INode> Nodes
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    // Load the Nodes in the model and store the cast separately
                    if (_nodes == null)
                    {
                        IQueryable<BasicFeedForwardNode> query = from n in _parent.Context.NodeSet
                                                                 where n.NetworkIdentifierFK == Identifier
                                                                 select n;
                        foreach (BasicFeedForwardNode node in query)
                        {
                            node.Initialize(this);
                            UnityContainer.BuildUp(node);
                        }

                        _nodes = new ObservableCollection<INode>(query.ToList<INode>());
                        _nodes.CollectionChanged += NodeCollectionChanged;
                    }
                    return _nodes;
                }
            }
        }

        /// <summary>
        /// Creates a BasicFeedForwardNode
        /// </summary>
        /// <returns>A BasicFeedForwardNode</returns>
        public INode CreateNode()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                INode toreturn = UnityContainer.Resolve<INode>(Parent.Name, new ParameterOverride("network", this));
                UnityContainer.BuildUp(toreturn);
                return toreturn;
            }
        }

        /// <summary>
        /// This is the callback method from the Observable Collection of nodes. This is done to keep
        /// the changes the user makes in sync with the database. This is always called by the Collection
        /// and should not be used by others
        /// </summary>
        /// <param name="sender">The object responsible for creating the event</param>
        /// <param name="e">The parameters telling what has changed</param>
        private void NodeCollectionChanged(Object sender, NotifyCollectionChangedEventArgs e)
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                switch (e.Action)
                {
                    case NotifyCollectionChangedAction.Add:
                        foreach (BasicFeedForwardNode toadd in e.NewItems)
                        {
                            _parent.Context.NodeSet.Add(toadd);
                            Node.Add(toadd);
                        }
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        try
                        {
                            foreach (BasicFeedForwardNode toremove in e.OldItems)
                            {
                                // Remove connections leading to or from the node                                
                                BasicFeedForwardNode removeCopy = toremove;
                                IQueryable<BasicFeedForwardConnection> queryConncections =
                                    from n in _parent.Context.ConnectionSet.OfType<BasicFeedForwardConnection>()
                                    where (n.InputNode == removeCopy || n.OutputNode == removeCopy)
                                    select n;

                                // buffer the stuff so we can delete
                                var coll = new Collection<BasicFeedForwardConnection>();
                                foreach (BasicFeedForwardConnection connection in queryConncections)
                                    coll.Add(connection);

                                foreach (BasicFeedForwardConnection conn in coll)
                                    _parent.Context.ConnectionSet.Remove(conn);

                                // Remove Node
                                _parent.Context.NodeSet.Remove(removeCopy);
                                Node.Remove(removeCopy);
                            }
                        }
                        finally
                        {
                            _connections = null;
                        }
                        break;
                }
            }
        }

        #endregion

        #region Connections

        private ObservableCollection<IConnection> _connections;
        private ObservableCollection<IInputConnection> _inputConnections;
        private ObservableCollection<IOutputConnection> _outputConnections;

        /// <summary>
        /// The collection containing all connections, including Input and Output Connections
        /// </summary>
        public ICollection<IConnection> Connections
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (_connections == null)
                        LoadConnections();

                    return _connections;
                }
            }
        }

        /// <summary>
        /// Collection containing only the Input Connections
        /// </summary>
        public ICollection<IInputConnection> InputConnections
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (_inputConnections == null)
                        LoadInputConnections();

                    return _inputConnections;
                }
            }
        }

        /// <summary>
        /// Collection containg only the Output Connections
        /// </summary>
        public ICollection<IOutputConnection> OutputConnections
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (_outputConnections == null)
                        LoadOutputConnections();

                    return _outputConnections;
                }
            }
        }

        /// <summary>
        /// Creates a BasicFeedForwardConnection
        /// </summary>
        /// <returns>A BasicFeedForwardConnect</returns>
        public IConnection CreateConnection()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                BasicFeedForwardConnection toreturn = UnityContainer.Resolve<BasicFeedForwardConnection>(Parent.Name, new ParameterOverride("network",this));
                return toreturn;
            }
        }

        /// <summary>
        /// Creates a BasicFeedForwardInputConnection using the Network Standard Input
        /// </summary>
        /// <returns></returns>
        public IInputConnection CreateInputConnection()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                var toreturn = new BasicFeedForwardInputConnection(this)
                                   {
                                       Logger = Logger,
                                       Weight = _inputOutputInitializationFunction.Calculate()
                                   };
                return toreturn;
            }
        }

        /// <summary>
        /// Creates a BasicFeedForwardOutputConnection using the Network Standard Output
        /// </summary>
        /// <returns></returns>
        public IOutputConnection CreateOutputConnection()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                var toreturn = new BasicFeedForwardOutputConnection(this)
                                   {
                                       Logger = Logger,
                                       Weight = _inputOutputInitializationFunction.Calculate()
                                   };
                return toreturn;
            }
        }

        /// <summary>
        /// Private Method to load connections belonging to this network into the Collection
        /// </summary>
        private void LoadConnections()
        {
            // pre-conditions
            Contract.Requires<InvalidOperationException>(_parent != null);
            Contract.Requires<InvalidOperationException>(_parent.Context != null);
            Contract.Requires<InvalidOperationException>(_parent.Context.NodeSet != null);
            Contract.Requires<InvalidOperationException>(_parent.Context.ConnectionSet != null);
            // post-conditions
            Contract.Ensures(_parent != null);
            Contract.Ensures(_parent.Context != null);
            Contract.Ensures(_parent.Context.NodeSet != null);
            Contract.Ensures(_parent.Context.ConnectionSet != null);
            Contract.Ensures(_connections != null);

            IQueryable<int> firstquery = from n in _parent.Context.NodeSet
                                         where n.NetworkIdentifierFK == Identifier
                                         select n.NodeId;

            IQueryable<BasicFeedForwardConnection> query = from t in _parent.Context.ConnectionSet
                                                           where
                                                               firstquery.Contains(t.Input_NodeId_FK.HasValue
                                                                                       ? t.Input_NodeId_FK.Value
                                                                                       : -1) ||
                                                               firstquery.Contains(t.Output_NodeId_FK.HasValue
                                                                                       ? t.Output_NodeId_FK.Value
                                                                                       : -1)
                                                           select t;

            _connections = new ObservableCollection<IConnection>(query.ToArray());
            _connections.CollectionChanged += ConnectionCollectionChanged;
        }

        /// <summary>
        /// Private method to load only the Input Connections
        /// </summary>
        private void LoadInputConnections()
        {
            if (_connections == null)
                LoadConnections();

            if (_connections != null)
            {
                IEnumerable<BasicFeedForwardInputConnection> query =
                    from t in _connections.OfType<BasicFeedForwardInputConnection>()
                    select t;

                _inputConnections = new ObservableCollection<IInputConnection>(query.ToArray());
            }
            _inputConnections.CollectionChanged += ConnectionCollectionChanged;
        }

        /// <summary>
        /// Private method to load only the output connections
        /// </summary>
        private void LoadOutputConnections()
        {
            if (_connections == null)
                LoadConnections();

            if (_connections != null)
            {
                IEnumerable<BasicFeedForwardOutputConnection> query =
                    from t in _connections.OfType<BasicFeedForwardOutputConnection>()
                    select t;

                _outputConnections = new ObservableCollection<IOutputConnection>(query.ToArray());
            }
            _outputConnections.CollectionChanged += ConnectionCollectionChanged;
        }

        /// <summary>
        /// Method triggered, if the user changes anything on the connection collection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConnectionCollectionChanged(Object sender, NotifyCollectionChangedEventArgs e)
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                switch (e.Action)
                {
                    case NotifyCollectionChangedAction.Add:
                        foreach (BasicFeedForwardConnection toadd in e.NewItems)
                        {
                            _parent.Context.ConnectionSet.Add(toadd);
                        }
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        foreach (BasicFeedForwardConnection toremove in e.OldItems)
                        {
                            _parent.Context.ConnectionSet.Remove(toremove);
                        }
                        break;
                }
                // TODO: clean up how this works - we don't need to reset everything all the time
                _connections = null;
                _inputConnections = null;
                _outputConnections = null;
            }
        }

        /// <summary>
        /// The input and output connection weights are defaulted to 1 by definition
        /// </summary>
        private readonly IInitializationFunction _inputOutputInitializationFunction = new UnitFunction();

        private IInitializationFunction _initializationFunction;
        /// <summary>
        /// This function ist to setup how the weights should be initialized
        /// </summary>
        public IInitializationFunction InitializationFunction
        {
            get { return _initializationFunction; }
            set
            {
                IParentNetwork initializationFunction = value as IParentNetwork;
                if (initializationFunction != null)
                    initializationFunction.ParentNetwork = this;
                else
                    throw new ArgumentException("Initialization Function must implement the IParentNetwork interface");

                InitializationFunctionInternal = FunctionLoader.TranslateStringFromInitializationFunction(value);
                _initializationFunction = value;
            }
        }

        private ILearningRule _learningRule;
        /// <summary>
        /// This defines the learning rule used 
        /// </summary>
        public ILearningRule LearningRule
        {
            get { return _learningRule; }
            set 
            { 
                Contract.Requires<ArgumentException>(value is IParentNetwork,Resources.LearningRuleIParentInterface);
                Contract.Ensures(value == _learningRule);
                IParentNetwork learningRule = value as IParentNetwork;
                learningRule.ParentNetwork = this;

                LearningRuleInternal = FunctionLoader.TranslateStringFromLearningRule(value);
                _learningRule = value;
            }
        }
        #endregion

        #region Inputs

        private IInput _input;

        /// <summary>
        /// List of Inputs to the Network
        /// </summary>
        public IInput Input
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (_input == null)
                    {
                        var formatter = new BinaryFormatter();
                        MemoryStream stream = null;
                        try
                        {
                            if (InputObject != null)
                            {
                                stream = new MemoryStream(InputObject);
                                _input = formatter.Deserialize(stream) as IInput;
                            }
                            //TODO: validate if the count of one dataset is equal the amount of input connections
                        }
                        finally
                        {
                            if (stream != null)
                                stream.Dispose();
                        }
                    }
                    return _input;
                }
            }
            set
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (value == null)
                    {
                        InputObject = null;
                        InputTypeName = null;
                        _input = null;
                    }
                    else
                    {
                        MemoryStream stream = null;
                        var formatter = new BinaryFormatter();
                        try
                        {
                            stream = new MemoryStream();
                            formatter.Serialize(stream, value);
                            InputTypeName = value.GetType().AssemblyQualifiedName;
                            InputObject = stream.ToArray();
                            _input = value;
                        }
                        finally
                        {
                            if (stream != null)
                                stream.Dispose();
                        }
                    }
                }
            }
        }

        #endregion

        #region Outputs

        private IOutput _output;

        public IOutput Output
        {
            get
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (_output == null)
                    {
                        var formatter = new BinaryFormatter();
                        MemoryStream stream = null;
                        try
                        {
                            if (OutputObject != null)
                            {
                                stream = new MemoryStream(OutputObject);
                                _output = formatter.Deserialize(stream) as IOutput;
                            }
                        }
                        finally
                        {
                            if (stream != null)
                                stream.Dispose();
                        }
                    }
                    return _output;
                }
            }
            set
            {
                using (Tracer.StartTrace("Ennf.NetworkModels"))
                {
                    if (value == null)
                    {
                        OutputObject = null;
                        OutputTypeName = null;
                        _output = null;
                    }
                    else
                    {
                        var formatter = new BinaryFormatter();
                        MemoryStream stream = null;
                        try
                        {
                            stream = new MemoryStream();
                            formatter.Serialize(stream, value);
                            OutputTypeName = value.GetType().AssemblyQualifiedName;
                            OutputObject = stream.ToArray();
                            _output = value;
                        }
                        finally
                        {
                            if (stream != null)
                                stream.Dispose();
                        }
                    }
                }
            }
        }

        #endregion

        #region Error and Target

        public IInput Target { get; set; }

        private IErrorFunction _errorFunction;
        /// <summary>
        /// This defines the error rule used
        /// </summary>
        public IErrorFunction ErrorFunction
        {
            get { return _errorFunction; }
            set 
            {
                IParentNetwork errorFunction = value as IParentNetwork;
                if (errorFunction != null)
                    errorFunction.ParentNetwork = this;
                else
                    throw new ArgumentException("The error function must implement the IParentNetwork");

                ErrorFunctionInternal = FunctionLoader.TranslateStringFromErrorFunction(value);
                _errorFunction = value;
            }
        }

        private IConvergenceSpeedFunction _convergenceSpeedFunction;
        /// <summary>
        /// This tracks the convergence measure function rule used
        /// </summary>
        public IConvergenceSpeedFunction ConvergenceSpeedFunction
        {
            get { return _convergenceSpeedFunction; } 
            set 
            { 
                IParentNetwork convergenceSpeedFunction = value as IParentNetwork;
                if (convergenceSpeedFunction != null)
                    convergenceSpeedFunction.ParentNetwork = this;
                else
                    throw new ArgumentException("The Convergence Speed Function must implement the IParentNetwork");

                ConvergenceSpeedFunctionInternal = FunctionLoader.TranslateStringFromConvergenceSpeedFunction(value);
                _convergenceSpeedFunction = value;
            }
        }
        #endregion

        #region Basic Network Operations

        /// <summary>
        /// Counts the amound of times Calculation was called
        /// </summary>
        public int CalculationIteration { get; private set; }

        /// <summary>
        /// Calculates the outputs based on the input values
        /// </summary>
        public void Calculate()
        {
            ValidateCalculationModel();

            using (Tracer.StartTrace("Ennf.NetworkModles.BasicFeedForward.Calculation"))
            {
                Dictionary<string, object> additionalInformation = new Dictionary<string, object>
                                                                       {
                                                                           {"CalculationIteration", ++CalculationIteration}
                                                                       };
                Tracer.LogWriter.Write("Parameters",additionalInformation);
                // If we are at the end of the Input Sets, we start from the beginning
                if (!Input.MoveNextInput())
                {
                    Input.ResetInput();
                    if (!Input.MoveNextInput())
                        throw new EnnfApplicationException("Could not reset Iterator");
                }
                Output.MoveNextResult();
                
                foreach (IOutputConnection outputConnection in OutputConnections)
                {
                    outputConnection.Calculate();
                }
            }
        }

        /// <summary>
        /// Counts the amount of times Learning was called
        /// </summary>
        public int LearningIteration { get; private set; }

        /// <summary>
        /// Adjusts the weights of the network according to the last calculations and the learning rule
        /// </summary>
        public void Learn()
        {
           ValidateLearningModel();

            using (Tracer.StartTrace("Ennf.NetworkModles.BasicFeedForward.Learning"))
            {
                if (!Target.MoveNextInput())
                {
                    // wrap around
                    Target.ResetInput();
                    if (!Target.MoveNextInput())
                        throw new EnnfApplicationException("Error wrapping around the Output Enumerator");
                }
                LearningRule.CalculateNewWeights(this);
                double error = ErrorFunction.Calculate();
                if (ConvergenceSpeedFunction != null)
                    ConvergenceSpeedFunction.RegisterError(error);
                Dictionary<string, object> additionalInformation = new Dictionary<string, object>
                                                                       {
                                                                           {"LearningIteration", ++LearningIteration},
                                                                           {"Error", error}
                                                                       };
                Tracer.LogWriter.Write("Parameters", additionalInformation);
            }
        }

        public void ValidateCompleteModel()
        {
            ValidateCalculationModel();
            ValidateLearningModel();
        }
        #endregion

        #region Network Properties
        public string Name { get; set; }
        public string Description { get; set; }
        #endregion

        #region Private Methods
        private void ValidateCalculationModel()
        {
            if (Input == null)
                throw new InvalidOperationException(Resources.InputMissing);
            if (Output == null)
                throw new InvalidOperationException(Resources.OutputMissing);
        }

        private void ValidateLearningModel()
        {
            if (ErrorFunction == null)
                throw new InvalidOperationException(Resources.ErrorFunctionMissing);
            if (LearningRule == null)
                throw new InvalidOperationException(Resources.LearningRuleFunctionMissing);
            if (LearningRule.LearningRate <= 0)
                throw new InvalidOperationException(Resources.LearningRateMissing);
            if (Target == null)
                throw new InvalidOperationException(Resources.TargetValuesMissing);
        }
        #endregion
    }
}