﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using Ennf.DbEntityAdapter;
using Ennf.Functions;
using Ennf.Functions.ActivationFunctions;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.NetworkModels.BasicFeedForward
{
    /// <summary>
    /// Represents a Node of a BasicFeedForard Model Network
    /// </summary>
    /// <remarks>Currently the fashion the INodeAdress is implemented is not nice, requires changing</remarks>
    public partial class BasicFeedForwardNode : INode, IParentNetwork
    {
        #region Tracing and Logging

        [Dependency]
        public LogWriter Logger { get; set; }

        [Dependency]
        public TraceManager Tracer { get; set; }

        [Dependency]
        public IXmlTypeLoader FunctionLoader { get; set; }

        #endregion

        #region Constructor
        public BasicFeedForwardNode(BasicFeedForwardNetwork network)
        {
            Contract.Requires<ArgumentNullException>(network != null);
            Contract.Ensures(ParentNetwork != null);
            Contract.Ensures(NetworkIdentifierFK == network.Identifier);
            Contract.Ensures(Address != null);
            Contract.Ensures(Address.AddressX == AddressX && Address.AddressY == AddressY && Address.AddressZ == AddressZ);

            Initialize(network);
        }


        internal void Initialize(BasicFeedForwardNetwork network)
        {
            ParentNetwork = network;
            NetworkIdentifierFK = network.Identifier;
            Address = new BasicFeedForwardNodeAddress { AddressX = AddressX, AddressY = AddressY, AddressZ = AddressZ };
        }

        #endregion

        #region INode Members

        private INodeAddress _address;
        [Dependency]
        public INodeAddress Address
        {
            get { return _address; } 
            set
            {
                Contract.Requires<ArgumentException>(value != null);
                Contract.Ensures(Address != null);
                Contract.Ensures(Address.AddressX == AddressX && Address.AddressY == AddressY && Address.AddressZ == AddressZ);

                _address = value;
                AddressX = _address.AddressX;
                AddressY = _address.AddressY;
                AddressZ = _address.AddressZ;
            }
        }

        private IActivationFunction _activationFunction;
        private bool _initialized;

        public IActivationFunction ActivationFunction
        {
            get
            {
                if (!_initialized)
                {
                    _activationFunction = ActivationFunctionInternal != null
                                              ? FunctionLoader.TranslateActivationFunctionFromString(
                                                  ActivationFunctionInternal)
                                              : new IdentityFunction();
                    _initialized = true;
                }
                return _activationFunction;
            } 
            set 
            { 
                ActivationFunctionInternal = FunctionLoader.TranslateStringFromActivationFunction(value);
                _activationFunction = value;
            }
        }

        #endregion

        #region Basic FeedForward Node Members

        public double SummationValue { get; private set; }

        public double OutputValue { get; private set; }

        public INetwork Network
        {
            get { return ParentNetwork; }
        }

        public double Calculate()
        {
            using (Tracer.StartTrace("Ennf.NetworkModles.BasicFeedForward.Calculation"))
            {
                Dictionary<string, object> additionalInformation = new Dictionary<string, object> { { "NodeID", NodeId } };

                // Get the connections
                var query = from n in Network.Connections
                            where n.OutputNode == this
                            select n;

                SummationValue = query.Cast<BasicFeedForwardConnection>().Sum(connection => connection.Calculate());
                OutputValue = ActivationFunction.Calculate(SummationValue);
                additionalInformation.Add("Summation Value",SummationValue);
                additionalInformation.Add("Return from Node", OutputValue);

                Tracer.LogWriter.Write("Parameters", "Ennf.NetworkModels.BasicFeedForward.Calculation", additionalInformation);
                return OutputValue;
            }
        }
        #endregion

        INetwork IParentNetwork.ParentNetwork
        {
            set
            {
                BasicFeedForwardNetwork network = value as BasicFeedForwardNetwork;
                Contract.Assert(network != null);
                Initialize(network);
            }
        }
    }
}