﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Data.Entity;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Xml.Serialization;
using Ennf.DbEntityAdapter;
using Ennf.Infrastructure;
using Ennf.Properties;
using Ennf.Utilities;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.NetworkModels.BasicFeedForward
{
    public class BasicFeedForwardPersistenceModelFileStore : IBasicFeedForwardPersistenceModel
    {
        [Dependency]
        public LogWriter Logger { get; set; }

        [Dependency]
        public IUnityContainer UnityContainer { get; set; }

        #region Implementation of IBasicFeedForwardPersistenceModel

        public IDbSet<BasicFeedForwardNetwork> NetworkSet { get; set; }

        public IDbSet<BasicFeedForwardNode> NodeSet { get; set; }

        public IDbSet<BasicFeedForwardConnection> ConnectionSet { get; set; }

        public IDbEntityEntry<TEntity> Entry<TEntity>(TEntity entity) where TEntity : class
        {
            throw new NotImplementedException();
        }

        public void LoadFile(string fileName)
        {
            NetworkSet = new DbSetAdapter<BasicFeedForwardNetwork>();
            NodeSet = new DbSetAdapter<BasicFeedForwardNode>();
            ConnectionSet = new DbSetAdapter<BasicFeedForwardConnection>();
            ReadFile(fileName);
        }

        public void ConnectDatabase(string connectionString)
        {
            throw new NotImplementedException();
        }

        public int SaveChanges()
        {
            // TODO: count the amount of changes were stored away
            return 0;
        }

        public void Close()
        {

        }
        #endregion

        #region Constructor
        public BasicFeedForwardPersistenceModelFileStore()
        {
            Logger = new NullLogger();
            UnityContainer = new UnityContainer();
        }
        
        public BasicFeedForwardPersistenceModelFileStore(string fileName)
        {
            Logger = new NullLogger();
            UnityContainer = new NullUnityContainer();
            LoadFile(fileName);
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Reads the XML File
        /// </summary>
        /// <param name="filename"></param>
        /// <remarks>This can be done smarter with Unity</remarks>
        private void ReadFile(string filename)
        {
            StreamReader reader = null;
            try
            {
                XmlSerializer deserializer = new XmlSerializer(typeof (EnnfType));
                reader = new StreamReader(filename, true);
                EnnfType des = deserializer.Deserialize(reader) as EnnfType;
                IXmlTypeLoader functionLoader = UnityContainer.Resolve<IXmlTypeLoader>();

                if (des == null)
                    throw new InvalidDataException(string.Format(CultureInfo.CurrentCulture, Resources.XmlFileInvalid,
                                                                 filename));

                foreach (EnnfTypeNetwork network in des.Network)
                {
                    //TODO: read the network type from the Xml
                    BasicFeedForwardNetwork newNetwork =
                        functionLoader.TranslateNetworkFromXml(network) as BasicFeedForwardNetwork;
                    if (newNetwork == null)
                        throw new EnnfNullReferenceException();
                    newNetwork.Identifier = int.Parse(network.Id, CultureInfo.InvariantCulture);
                    UnityContainer.BuildUp(newNetwork);
                    NetworkSet.Add(newNetwork);
                    // now load all the nodes
                    if (network.Nodes != null)
                    {
                        foreach (EnnfTypeNetworkNodesNode node in network.Nodes.Node)
                        {
                            BasicFeedForwardNode insertNode =
                                functionLoader.TranslateNodeFromXml(node) as BasicFeedForwardNode;
                            if (insertNode == null)
                                throw new EnnfNullReferenceException();

                            UnityContainer.BuildUp(insertNode);
                            insertNode.NetworkIdentifierFK = newNetwork.Identifier;
                            insertNode.NodeId = int.Parse(node.NodeId, CultureInfo.InvariantCulture);
                            insertNode.Address = new BasicFeedForwardNodeAddress
                                {
                                    AddressX = int.Parse(node.AddressX, CultureInfo.InvariantCulture),
                                    AddressY = int.Parse(node.AddressY, CultureInfo.InvariantCulture),
                                    AddressZ = int.Parse(node.AddressZ, CultureInfo.InvariantCulture)
                                };
                            NodeSet.Add(insertNode);
                        }
                    }
                    // load all the connections
                    if (network.Connections != null)
                    {
                        if (network.Connections.Connections != null)
                        {
                            foreach (
                                EnnfTypeNetworkConnectionsConnectionsConnection connection in
                                    network.Connections.Connections.Connection)
                            {
                                BasicFeedForwardConnection insertConnection =
                                    functionLoader.TranslateConnectionFromXml(connection) as BasicFeedForwardConnection;

                                if (insertConnection == null)
                                    throw new EnnfNullReferenceException(Resources.ConnectionTypeNull);

                                insertConnection.ConnectionId = int.Parse(connection.ConnectionId,
                                                                          CultureInfo.InvariantCulture);

                                UnityContainer.BuildUp(insertConnection);
                                if (connection.InputNodeId.Length > 0)
                                    insertConnection.InputNode =
                                        NodeSet.Single(
                                            node =>
                                            node.NodeId ==
                                            int.Parse(connection.InputNodeId, CultureInfo.InvariantCulture));
                                if (connection.OutputNodeId.Length > 0)
                                    insertConnection.OutputNode =
                                        NodeSet.Single(
                                            node =>
                                            node.NodeId ==
                                            int.Parse(connection.OutputNodeId, CultureInfo.InvariantCulture));
                                ConnectionSet.Add(insertConnection);

                            }
                        }

                        if (network.Connections.InputConnections != null)
                        {
                            foreach (
                                EnnfTypeNetworkConnectionsInputConnectionsInputConnection connection in
                                    network.Connections.InputConnections.InputConnection)
                            {
                                BasicFeedForwardInputConnection insertConnection =
                                    functionLoader.TranslateInputConnectionFromXml(connection) as
                                    BasicFeedForwardInputConnection;

                                if (insertConnection == null)
                                    throw new EnnfNullReferenceException(Resources.ConnectionTypeNull);

                                insertConnection.ConnectionId = int.Parse(connection.ConnectionId,
                                                                          CultureInfo.InvariantCulture);

                                UnityContainer.BuildUp(insertConnection);

                                if (connection.OutputNodeId.Length > 0)
                                    insertConnection.OutputNode =
                                        NodeSet.Single(
                                            node =>
                                            node.NodeId ==
                                            int.Parse(connection.OutputNodeId, CultureInfo.InvariantCulture));
                                ConnectionSet.Add(insertConnection);

                            }
                        }

                        if (network.Connections != null)
                        {
                            if (network.Connections.OutputConnections != null)
                            {
                                foreach (
                                    EnnfTypeNetworkConnectionsOutputConnectionsOutputConnection connection in
                                        network.Connections.OutputConnections.OutputConnection)
                                {
                                    BasicFeedForwardOutputConnection insertConnection =
                                        functionLoader.TranslateOutputConnectionFromXml(connection) as
                                        BasicFeedForwardOutputConnection;

                                    if (insertConnection == null)
                                        throw new EnnfNullReferenceException(Resources.ConnectionTypeNull);

                                    insertConnection.ConnectionId = int.Parse(connection.ConnectionId,
                                                                              CultureInfo.InvariantCulture);

                                    UnityContainer.BuildUp(insertConnection);
                                    if (connection.InputNodeId.Length > 0)
                                        insertConnection.InputNode =
                                            NodeSet.Single(
                                                node =>
                                                node.NodeId ==
                                                int.Parse(connection.InputNodeId, CultureInfo.InvariantCulture));

                                    ConnectionSet.Add(insertConnection);

                                }
                            }
                        }
                    }
                }
                
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                    reader.Dispose();
                }
            }
        }

        /*private void WriteFile()
        {
            
        }*/
        #endregion

        
    }
}
