﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using Ennf.DbEntityAdapter;
using Ennf.Infrastructure;
using Ennf.NetworkModels.BasicFeedForward;
using Ennf.Properties;
using Ennf.Utilities;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.Unity;

namespace Ennf.NetworkModels
{
    /// <summary>
    /// Central Management Object for the BasicFeedForward Neural Network Model
    /// </summary>
    public class BasicFeedForwardModel : INetworkModel
    {
        #region Persistence Data Context

        public IPersistenceModel PersistenceModel { get; set; }
        internal IBasicFeedForwardPersistenceModel Context { get { return PersistenceModel as IBasicFeedForwardPersistenceModel; } }

        #endregion

        #region Properties
        [Dependency]
        public LogWriter Logger { get; set; }
        [Dependency]
        public TraceManager Tracer { get; set; }

        [Dependency]
        public IUnityContainer UnityContainer { get; set; }

        [Dependency]
        public IXmlTypeLoader FunctionLoader { get; set; }

        public string Name { get { return Resources.BasicFeedForwardModelType; } }
        #endregion

        #region Constructor

        public BasicFeedForwardModel()
        {
            Logger = new NullLogger();
            UnityContainer = new NullUnityContainer();
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="context">The database interface</param>
        public BasicFeedForwardModel(IBasicFeedForwardPersistenceModel context):this()
        {
            PersistenceModel = context;
        }

        #endregion

        #region Network Operations

        /// <summary>
        /// Creates a empty BasicFeedForward Neural Network 
        /// </summary>
        /// <remarks>This method is a basic factory method until a better factory setup is developed</remarks>
        /// <returns>An instance of a Neural Network</returns>
        public INetwork CreateNetwork()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                BasicFeedForwardNetwork toreturn = Context.NetworkSet.Create();
                UnityContainer.BuildUp(toreturn, Name);
                toreturn.Parent = this;
                toreturn.Identifier = -1;
                toreturn.NetworkModel = toreturn.GetType().Name;

                Context.NetworkSet.Add(toreturn);
                Context.Entry(toreturn).State = EntityState.Added;

                return toreturn;
            }
        }

        /// <summary>
        /// Loads a BasicFeedForward Neural Network from the underlying database
        /// </summary>
        /// <param name="identifier">Identifier of the Network</param>
        /// <returns>The loaded Network, currently only the BasicFeedForwardNetwork</returns>
        public INetwork LoadNetwork(int identifier)
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                Dictionary<string, object> additionalInformation = new Dictionary<string, object>
                                                                       {{"identifier", identifier}};
                Tracer.LogWriter.Write("BasicFeedForwardModel.LoadNetwork(int) Parameters", "", 1, 100,
                                              TraceEventType.Verbose, "ennf", additionalInformation);

                IQueryable<BasicFeedForwardNetwork> query = from n in Context.NetworkSet
                                                            where n.Identifier == identifier
                                                            select n;

                // we didn't find anything, we throw an exception as this was to be specific
                if (!query.Any())
                    throw new EnnfApplicationException(String.Format(CultureInfo.InvariantCulture, "Network with Id {0} not found", identifier));
                BasicFeedForwardNetwork toreturn = query.Single();
                UnityContainer.BuildUp(toreturn);
                toreturn.Parent = this;
                SetupNetwork(toreturn);
                return toreturn;
            }
        }

        /// <summary>
        /// Loads the list of all BasicFeedForward Neural Networks from the underlying database
        /// </summary>
        /// <returns>The list of networks</returns>
        public Collection<INetwork> LoadNetwork()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                IQueryable<BasicFeedForwardNetwork> query = from n in Context.NetworkSet
                                                            select n;

                List<BasicFeedForwardNetwork> toreturn = query.ToList();
                foreach (BasicFeedForwardNetwork network in toreturn)
                {
                    UnityContainer.BuildUp(network);
                    network.Parent = this;
                    SetupNetwork(network);
                }

                return new ObservableCollection<INetwork>(toreturn.ToList<INetwork>());
            }
        }

        private void SetupNetwork(BasicFeedForwardNetwork network)
        {
            if (network.InitializationFunctionInternal != null)
            {
                network.InitializationFunction =
                    FunctionLoader.TranslateInitializationFunctionFromString(network.InitializationFunctionInternal);
            }
            if (network.ErrorFunctionInternal != null)
            {
                network.ErrorFunction = FunctionLoader.TranslateErrorFunctionFromString(network.ErrorFunctionInternal);
            }
            if (network.LearningRuleInternal != null)
            {
                network.LearningRule = FunctionLoader.TranslateLearningRuleFromString(network.LearningRuleInternal);
            }
            if (network.ConvergenceSpeedFunctionInternal != null)
            {
                network.ConvergenceSpeedFunction =
                    FunctionLoader.TranslateConvergenceSpeedFunctionFromString(network.ConvergenceSpeedFunctionInternal);
            }
        }

        /// <summary>
        /// Saves the BasicFeedForward Neural Network changes to the underlying database
        /// </summary>
        public virtual void Save()
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                // For some reason the update to the nodes is acknowledge after a full load of 
                // network - need to understand the process here
                IQueryable<BasicFeedForwardNetwork> query = from n in Context.NetworkSet
                                                            select n;
                query.ToList();
                

                Context.SaveChanges();
            }
        }

        /// <summary>
        /// Deletes the BasicFeedForward Neural Network
        /// </summary>
        /// <param name="deletee">The Neural Network to delete</param>
        /// <exception cref="ArgumentException">Thrown if the Network to delete is not of type BasicFeedForwardNetwork</exception>
        public void DeleteNetwork(INetwork deletee)
        {
            using (Tracer.StartTrace("Ennf.NetworkModels"))
            {
                var additionalInformation = new Dictionary<string, object> {{"deletee", deletee}};
                Tracer.LogWriter.Write("BasicFeedForwardModel.DeleteNetwork(INetwork) parameters", "", 1, 100,
                                              TraceEventType.Verbose, "ennf", additionalInformation);

                var converted = deletee as BasicFeedForwardNetwork;

                if (converted == null)
                    throw new ArgumentException("The network type is unknown");

                Context.NetworkSet.Remove(converted);
            }
        }

        

        #endregion
    }
}