﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using Ennf.Functions;
using Ennf.Functions.ActivationFunctions;
using Ennf.Properties;

namespace Ennf.Utilities
{
    /// <summary>
    /// Helper class to help creating networks
    /// </summary>
    public class LayerHelper
    {
        private readonly INetwork _parentNetwork;
        private readonly List<INode> _inputNodes = new List<INode>();
        private readonly List<List<INode>> _layers = new List<List<INode>>(); 

        public LayerHelper(INetwork network)
        {
            _parentNetwork = network;
        }

        [ContractInvariantMethod]
        private void Invariants()
        {
            Contract.Invariant(_inputNodes != null);
            Contract.Invariant(_layers != null);
        }

        public void CreateInputs(int inputCount)
        {
            for (int i = 0; i < inputCount; i++)
            {
                INode node = _parentNetwork.CreateNode();
                IInputConnection connection = _parentNetwork.CreateInputConnection();
                node.Address.AddressX = i;
                node.Address.AddressY = 0;
                node.Address.AddressZ = 0;

                connection.OutputNode = node;
                _parentNetwork.Nodes.Add(node);
                _parentNetwork.Connections.Add(connection);
                _inputNodes.Add(node);
            }
        }

        public int CreateLayer(int nodeCount)
        {
            Contract.Requires(nodeCount > 0);
            Contract.Ensures(Contract.Result<int>() >= 0);
            Contract.Ensures(Contract.Result<int>() == _layers.Count - 1);
            IActivationFunction function = new LogisticSigmoidFunction();
            return CreateLayer(nodeCount, function);
        }

        public int CreateLayer(int nodeCount,IActivationFunction function)
        {
            Contract.Ensures(Contract.Result<int>() >= 0);
            Contract.Ensures(Contract.Result<int>() == _layers.Count - 1);

            _layers.Add(new List<INode>());
            int layer = _layers.Count-1;
            for (int i = 0; i < nodeCount; i++)
            {
                INode node = _parentNetwork.CreateNode();
                node.Address.AddressX = i;
                node.Address.AddressY = layer+1;
                node.ActivationFunction = function;
                _parentNetwork.Nodes.Add(node);
                _layers[layer].Add(node);
            }
            return layer;
        }

        public void ConnectLayerToInput(int layer)
        {
            Contract.Requires<ArgumentOutOfRangeException>(layer >= 0);
            
            if (layer > _layers.Count-1)
                throw new ArgumentOutOfRangeException("layer");

            foreach (INode node in _layers[layer])
            {
                foreach (INode inputNode in _inputNodes)
                {
                    IConnection connection = _parentNetwork.CreateConnection();
                    connection.InputNode = inputNode;
                    connection.OutputNode = node;
                    _parentNetwork.Connections.Add(connection);
                }
            }
        }

        public void ConnectLayerToLayer(int inputLayer,int outputLayer)
        {
            if (inputLayer > _layers.Count-1 || outputLayer > _layers.Count-1)
                throw new ArgumentOutOfRangeException("inputLayer or outputLayer");

            foreach (INode inputNode in _layers[inputLayer])
            {
                foreach (INode outputNode in _layers[outputLayer])
                {
                    IConnection connection = _parentNetwork.CreateConnection();
                    connection.InputNode = inputNode;
                    connection.OutputNode = outputNode;
                    _parentNetwork.Connections.Add(connection);
                }
            }
        }

        public void CreateOutput(int outputLayer)
        {
            Contract.Requires<ArgumentOutOfRangeException>(outputLayer >= 0);

            if (_layers.Count == 0)
                throw new InvalidOperationException(Resources.LayerHelperMissingLayers);
            if (outputLayer > _layers.Count - 1)
                throw new ArgumentOutOfRangeException("outputLayer");

            foreach (INode node in _layers[outputLayer])
            {
                IOutputConnection outputConnection = _parentNetwork.CreateOutputConnection();
                outputConnection.InputNode = node;
                _parentNetwork.OutputConnections.Add(outputConnection);
            }
        }
    }
}
