﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Diagnostics.Contracts;
using System.Globalization;
using System.IO;
using System.Xml.Serialization;
using Ennf.DbEntityAdapter;
using Ennf.Functions;
using Ennf.Functions.ConvergenceSpeedFunctions;
using Ennf.Infrastructure;
using Ennf.LearningRules;
using Ennf.NetworkModels.BasicFeedForward;
using Ennf.Properties;
using Microsoft.Practices.Unity;

namespace Ennf.Utilities
{
    /// <summary>
    /// Methods to serialize and deserialize object instances
    /// </summary>
    public class XmlTypeLoader : IXmlTypeLoader
    {
        [Dependency]
        public IUnityContainer UnityContainer { get; set; }
        /// <summary>
        /// Instantiates a serialized object
        /// </summary>
        /// <typeparam name="T">Type to instantiate</typeparam>
        /// <param name="serializedData">String containing serialized data</param>
        /// <returns>The instantiated object</returns>
        public T LoadObject<T>(string serializedData) where T:class
        {
            Contract.Requires<ArgumentNullException>(serializedData != null);
            StringReader reader = null;
            try
            {
                reader = new StringReader(serializedData);

                XmlSerializer serializer = new XmlSerializer(typeof (T));
                T toreturn = serializer.Deserialize(reader) as T;
                return toreturn;
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public XmlTypeLoader()
        {
            UnityContainer = new NullUnityContainer();
        }

        public INetwork TranslateNetworkFromXml(EnnfTypeNetwork network)
        {
            Contract.Requires<ArgumentNullException>(network != null);
            BasicFeedForwardNetwork toreturn = InstantiateObject<INetwork>(network.Type) as BasicFeedForwardNetwork;
            UnityContainer.BuildUp(toreturn);
            if (toreturn == null)
                throw new EnnfNullReferenceException("Error in the unity container configuration, toreturn is null");

            toreturn.Name = network.Name;
            toreturn.Description = network.Description;
            toreturn.LearningRule = TranslateLearningRuleFromXml(network.LearningRule);
            toreturn.ConvergenceSpeedFunction =
                TranslateConvergenceSpeedFunctionFromXml(network.ConvergenceSpeedFunction);
            toreturn.ErrorFunction = TranslateErrorFunctionFromXml(network.ErrorFunction);
            toreturn.InitializationFunction = TranslateInitializationFunctionFromXml(network.InitializationFunction);

            return toreturn;
        }

        #region Learning Rule

        public ILearningRule TranslateLearningRuleFromXml(LearningRuleType learningRule)
        {
            Contract.Requires<ArgumentNullException>(learningRule != null);
            ILearningRule toreturn = InstantiateObject<ILearningRule>(learningRule.Type);

            if (learningRule.LearningRateSpecified)
                toreturn.LearningRate = learningRule.LearningRate;

            UnityContainer.BuildUp(toreturn);
            return toreturn;
        }

        public ILearningRule TranslateLearningRuleFromString(string learningRule)
        {
            Contract.Requires<ArgumentNullException>(learningRule != null);
            XmlSerializer deserializer = new XmlSerializer(typeof(LearningRuleType), null, new[] { typeof(double) }, new XmlRootAttribute("LearningRule"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringReader reader = null;

            try
            {
                reader = new StringReader(learningRule);
                return TranslateLearningRuleFromXml(deserializer.Deserialize(reader) as LearningRuleType);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public string TranslateStringFromLearningRule(ILearningRule learningRule)
        {
            Contract.Requires<ArgumentNullException>(learningRule != null);
            XmlSerializer serializer = new XmlSerializer(typeof(LearningRuleType), null, new[] { typeof(double) }, new XmlRootAttribute("LearningRule"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringWriter writer = null;

            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);

                serializer.Serialize(writer, TranslateXmlFromLearningRule(learningRule));
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        public LearningRuleType TranslateXmlFromLearningRule(ILearningRule learningRule)
        {
            Contract.Requires<ArgumentNullException>(learningRule != null);
            LearningRuleType toreturn = new LearningRuleType
                                            {
                                                LearningRate = learningRule.LearningRate,
                                                LearningRateSpecified = (learningRule.LearningRate != 0.0),
                                                Type = learningRule.GetType().AssemblyQualifiedName
                                            };

            return toreturn;
        }

        #endregion

        #region Convergence Speed

        public IConvergenceSpeedFunction TranslateConvergenceSpeedFunctionFromXml(ConvergenceSpeedFunctionType convergenceSpeedFunction)
        {
            Contract.Requires<ArgumentNullException>(convergenceSpeedFunction != null);
            IConvergenceSpeedFunction toreturn =
                InstantiateObject<IConvergenceSpeedFunction>(convergenceSpeedFunction.Type);

            // Type-specific setup - we'll have to see what we do here about configuring this
            MovingAverageConvergenceSpeed movingAverageConvergenceSpeed = toreturn as MovingAverageConvergenceSpeed;
            Contract.Assert(movingAverageConvergenceSpeed != null);
            if (convergenceSpeedFunction.Decay != null)
                (movingAverageConvergenceSpeed).Decay = int.Parse(convergenceSpeedFunction.Decay,CultureInfo.InvariantCulture);

            return toreturn;
        }

        public IConvergenceSpeedFunction TranslateConvergenceSpeedFunctionFromString(string convergenceSpeedFunction)
        {
            XmlSerializer deserializer = new XmlSerializer(typeof(ConvergenceSpeedFunctionType), null, new[] { typeof(int) }, new XmlRootAttribute("ConvergenceSpeedFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringReader reader = null;

            try
            {
                reader = new StringReader(convergenceSpeedFunction);

                return
                    TranslateConvergenceSpeedFunctionFromXml(
                        deserializer.Deserialize(reader) as ConvergenceSpeedFunctionType);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public string TranslateStringFromConvergenceSpeedFunction(IConvergenceSpeedFunction convergenceSpeedFunction)
        {
            Contract.Requires<ArgumentNullException>(convergenceSpeedFunction != null);
            XmlSerializer serializer = new XmlSerializer(typeof(ConvergenceSpeedFunctionType), null, new[] { typeof(int) }, new XmlRootAttribute("ConvergenceSpeedFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringWriter writer = null;
            
            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);
                serializer.Serialize(writer, TranslateXmlFromConvergenceSpeedFunction(convergenceSpeedFunction));
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
            
        }

        public ConvergenceSpeedFunctionType TranslateXmlFromConvergenceSpeedFunction(IConvergenceSpeedFunction convergenceSpeedFunction)
        {
            Contract.Requires<ArgumentNullException>(convergenceSpeedFunction != null);
            ConvergenceSpeedFunctionType toreturn = new ConvergenceSpeedFunctionType
                                                        {
                                                            Type = convergenceSpeedFunction.GetType().AssemblyQualifiedName
                                                        };

            return toreturn;
        }

        #endregion

        #region Error Function

        public IErrorFunction TranslateErrorFunctionFromXml(ErrorFunctionType errorFunction)
        {
            Contract.Requires<ArgumentNullException>(errorFunction != null);
            return InstantiateObject<IErrorFunction>(errorFunction.Type);
        }

        public IErrorFunction TranslateErrorFunctionFromString(string errorFunction)
        {
            Contract.Requires<ArgumentNullException>(errorFunction != null);
            XmlSerializer deserializer = new XmlSerializer(typeof(ErrorFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("ErrorFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringReader reader = null;

            try
            {
                reader = new StringReader(errorFunction);
                return TranslateErrorFunctionFromXml(deserializer.Deserialize(reader) as ErrorFunctionType);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public string TranslateStringFromErrorFunction(IErrorFunction errorFunction)
        {
            Contract.Requires<ArgumentNullException>(errorFunction != null);
            XmlSerializer serializer = new XmlSerializer(typeof(ErrorFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("ErrorFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringWriter writer = null;

            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);

                serializer.Serialize(writer, TranslateXmlFromErrorFunction(errorFunction));
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        public ErrorFunctionType TranslateXmlFromErrorFunction(IErrorFunction errorFunction)
        {
            Contract.Requires<ArgumentNullException>(errorFunction !=null);
            ErrorFunctionType toreturn = new ErrorFunctionType
                                             {
                                                 Type = errorFunction.GetType().AssemblyQualifiedName
                                             };

            return toreturn;
        }

        #endregion

        #region Initialization Function

        public IInitializationFunction TranslateInitializationFunctionFromXml(InitializationFunctionType initializationFunction)
        {
            Contract.Requires<ArgumentNullException>(initializationFunction != null);
            IInitializationFunction toreturn = InstantiateObject<IInitializationFunction>(initializationFunction.Type);

            toreturn.Maximum = initializationFunction.Maximum;
            toreturn.Minimum = initializationFunction.Minimum;

            return toreturn;
        }

        public IInitializationFunction TranslateInitializationFunctionFromString(string initializationFunction)
        {
            XmlSerializer deserializer = new XmlSerializer(typeof(InitializationFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("InitializationFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringReader reader = null;

            try
            {
                reader = new StringReader(initializationFunction);
                return TranslateInitializationFunctionFromXml(deserializer.Deserialize(reader) as InitializationFunctionType);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        public string TranslateStringFromInitializationFunction(IInitializationFunction initializationFunction)
        {
            Contract.Requires<ArgumentNullException>(initializationFunction != null);
            XmlSerializer serializer = new XmlSerializer(typeof(InitializationFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("InitializationFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");

            StringWriter writer = null;
            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);

                serializer.Serialize(writer, TranslateXmlFromInitializationFunction(initializationFunction));
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        public InitializationFunctionType TranslateXmlFromInitializationFunction(IInitializationFunction initializationFunction)
        {
            Contract.Requires<ArgumentNullException>(initializationFunction != null);
            InitializationFunctionType toreturn = new InitializationFunctionType
                                                      {
                                                          Maximum = initializationFunction.Maximum,
                                                          Minimum = initializationFunction.Minimum,
                                                          Type = initializationFunction.GetType().AssemblyQualifiedName
                                                      };

            return toreturn;
        }

        #endregion

        #region Activation Function

        public IActivationFunction TranslateActivationFunctionFromXml(ActivationFunctionType activationFunction)
        {
            Contract.Requires<ArgumentNullException>(activationFunction != null);
            return InstantiateObject<IActivationFunction>(activationFunction.Type);
        }

        public IActivationFunction TranslateActivationFunctionFromString(string activationFunction)
        {
            XmlSerializer deserializer = new XmlSerializer(typeof(ActivationFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("ActivationFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringReader reader = null;

            try
            {
                reader = new StringReader(activationFunction);
                return TranslateActivationFunctionFromXml(deserializer.Deserialize(reader) as ActivationFunctionType);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
            
        }

        public string TranslateStringFromActivationFunction(IActivationFunction activationFunction)
        {
            Contract.Requires<ArgumentNullException>(activationFunction != null);
            XmlSerializer serializer = new XmlSerializer(typeof(ActivationFunctionType), null, new[] { typeof(double) }, new XmlRootAttribute("ActivationFunction"), "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
            StringWriter writer = null;

            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);
                serializer.Serialize(writer, TranslateXmlFromActivationFunction(activationFunction));
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        public ActivationFunctionType TranslateXmlFromActivationFunction(IActivationFunction activationFunction)
        {
            Contract.Requires<ArgumentNullException>(activationFunction != null);
            ActivationFunctionType toreturn = new ActivationFunctionType
                                                  {
                                                      Type = activationFunction.GetType().AssemblyQualifiedName
                                                  };
            return toreturn;
        }

        #endregion

        public IConnection TranslateConnectionFromXml(EnnfTypeNetworkConnectionsConnectionsConnection connection)
        {
            Contract.Requires<ArgumentNullException>(connection != null);
            return InstantiateObject<IConnection>(connection.Type);
        }

        public IInputConnection TranslateInputConnectionFromXml(EnnfTypeNetworkConnectionsInputConnectionsInputConnection inputConnection)
        {
            Contract.Requires<ArgumentNullException>(inputConnection != null);
            return InstantiateObject<IInputConnection>(inputConnection.Type);
        }

        public IOutputConnection TranslateOutputConnectionFromXml(EnnfTypeNetworkConnectionsOutputConnectionsOutputConnection outputConnection)
        {
            Contract.Requires<ArgumentNullException>(outputConnection != null);
            return InstantiateObject<IOutputConnection>(outputConnection.Type);
        }

        public INode TranslateNodeFromXml(EnnfTypeNetworkNodesNode node)
        {
            Contract.Requires<ArgumentNullException>(node != null);
            BasicFeedForwardNode toreturn = InstantiateObject<INode>(node.Type) as BasicFeedForwardNode;
            Contract.Assume(toreturn != null);
            UnityContainer.BuildUp(toreturn);
            toreturn.ActivationFunction = TranslateActivationFunctionFromXml(node.ActivationFunction);

            return toreturn;
        }

        private static T InstantiateObject<T>(string type) where T:class 
        {
            Type toinstant = Type.GetType(type);
            if (toinstant == null)
                throw new TypeLoadException(String.Format(CultureInfo.CurrentCulture, Resources.TypeUnknown, type));

            T toreturn = Activator.CreateInstance(toinstant) as T;
            if (toreturn == null)
                throw new TypeLoadException(String.Format(CultureInfo.CurrentCulture, Resources.TypeUnknown, type));

            return toreturn;
        }

        /// <summary>
        /// Serializes a object
        /// </summary>
        /// <param name="toserialize">Object to serialize</param>
        /// <returns>String representing object</returns>
        public string UnloadObject(object toserialize)
        {
            Contract.Requires<ArgumentNullException>(toserialize != null);
            StringWriter writer = null;

            try
            {
                writer = new StringWriter(CultureInfo.InvariantCulture);

                XmlSerializer serializer = new XmlSerializer(toserialize.GetType(),
                                                             "http://ralphovi.org/BasicFeedForwardXmlSchema.xsd");
                serializer.Serialize(writer, toserialize);
                return writer.ToString();
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }
    }
}
