﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.ComponentModel;
using Ennf;
using EnnfDesigner.BasicFeedForward.Properties;
using Microsoft.Practices.Prism.ViewModel;

namespace EnnfDesigner.BasicFeedForward.Models
{

    public class BasicFeedForwardModel : NotificationObject 
    {
        private BackgroundWorker _backgroundTrainer;
        private object _cancelLock = new object();

        #region Public Methods
        public void Train()
        {
            if (WorkInProgress)
                throw new InvalidOperationException(Resources.WorkAlreadyInProgress);

            _backgroundTrainer = new BackgroundWorker();
            _backgroundTrainer.DoWork += BackgroundTraining;
            _backgroundTrainer.RunWorkerCompleted += BackgroundTrainingCompleted;
            _backgroundTrainer.WorkerSupportsCancellation = true;

            _backgroundTrainer.RunWorkerAsync();

        }

        public void CancelWork()
        {
            if (!WorkInProgress)
                return;
            lock (_cancelLock)
            {
                _backgroundTrainer.CancelAsync();
            }
        }
        #endregion

        #region Public Properties

        public event EventHandler CalculationCompletedEvent;

        public INetwork Network { get; set; }

        public int Iteration { get; private set; }

        public double Error { get; private set; }

        public bool WorkInProgress
        {
            get
            {
                return _backgroundTrainer != null && _backgroundTrainer.IsBusy;
            }
        }

        public double MaxError { get; set; }

        public double Convergence { get; set; }

        #endregion

        #region Internal Methods
        void BackgroundTraining(object sender, DoWorkEventArgs e)
        {
            // do training
            double convergence = 1.0;
            Error = 1.0;
            while (convergence > Convergence && Error > MaxError)
            {
                Network.Calculate();
                Network.Learn();
                Error = Network.ErrorFunction.Calculate();
                convergence = Network.ConvergenceSpeedFunction.Calculate();
                Iteration++;

                // Update Screen every 100 iteraions
                if (Iteration % 100 == 0)
                {
                    RaisePropertyChanged(() => Iteration);
                    RaisePropertyChanged(() => Error);
                }
                lock (_cancelLock)
                {
                    if (_backgroundTrainer.CancellationPending)
                        return;
                }
            }
        }

        void BackgroundTrainingCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (CalculationCompletedEvent != null)
                CalculationCompletedEvent(this, e);
        }
        #endregion
    }
}
