﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Windows.Input;
using Ennf;
using Ennf.Functions;
using Ennf.Infrastructure;
using Ennf.LearningRules;
using EnnfDesigner.BasicFeedForward.Properties;
using EnnfDesigner.Common.Events;
using EnnfDesigner.Common.ShellServices;
using EnnfDesigner.Common.Utilities;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using EnnfDesigner.BasicFeedForward.Models;
using Microsoft.Practices.Unity;

namespace EnnfDesigner.BasicFeedForward.ViewModels
{
    public class TrainingConsoleViewModel : NotificationObject
    {
        #region Model Setup & Definition
        private readonly BasicFeedForwardModel _model;
        private readonly IEventAggregator _eventAggregator;
        private readonly IList<FunctionListDescription> _errorFunctions;
        private readonly IList<FunctionListDescription> _learningRules;
        private readonly IUnityContainer _container;
        private readonly IShellService _shellService;
        private string _targetFile;
        private string _trainingFile;
        private double _learningRate;
        
        private readonly IList<FunctionListDescription> _convergenceFunctions;

        public TrainingConsoleViewModel(IUnityContainer container, IEventAggregator eventAggregator)
        {
            // Init values
            ConvergenceDecayCount = 10;
            _model = new BasicFeedForwardModel();
            // Setup the wiring for integration with the Shell
            _container = container;
            _eventAggregator = eventAggregator;
            _shellService = _container.Resolve<IShellService>();
            _eventAggregator.GetEvent<NetworkSelectedEvent>().Subscribe(OnNetworkChanged);
            _model.PropertyChanged += ModelOnPropertyChanged;
            _model.CalculationCompletedEvent += ModelCalculationCompletedEvent;
            
            // Setup the DropDown Lists
            _errorFunctions = InitializeFunctionList(FunctionFinder.FindLoadedErrorFunctions);
            _learningRules = InitializeLearningList(FunctionFinder.FindLoadedLearningRules);
            _convergenceFunctions = InitializeFunctionList(FunctionFinder.FindLoadedConvergenceSpeedFunctions);

            //Initialize the commands
            LoadTargetFile = new DelegateCommand<string>(OnLoadTargetFile, param => !_model.WorkInProgress);
            LoadTrainingFile = new DelegateCommand<string>(OnLoadTrainingFile, param => !_model.WorkInProgress);
            TrainSingleStep = new DelegateCommand(OnSingleTraining, ()=> !_model.WorkInProgress);
            Train = new DelegateCommand(OnTraining, () => !WorkInProgress);
            ResetTraining = new DelegateCommand(OnResetTraining, () => WorkInProgress);}

        

        private IList<FunctionListDescription> InitializeFunctionList (Func<IList<Type>> findFunction ) 
        {
            IList<FunctionListDescription> functionLists = new List<FunctionListDescription>();
            foreach (Type type in findFunction.Invoke())
            {
                IFunction functionInstance = Activator.CreateInstance(type) as IFunction;
                if (functionInstance == null)
                    throw new ApplicationException(string.Format(Resources.FoundFunctionNotIFunction,type.FullName));
                functionLists.Add(new FunctionListDescription {Name = functionInstance.Name, Type = type});
            }

            return functionLists;
        }

        private IList<FunctionListDescription> InitializeLearningList(Func<IList<Type>> findFunction)
        {
            IList<FunctionListDescription> functionLists = new List<FunctionListDescription>();
            foreach (Type type in findFunction.Invoke())
            {
                ILearningRule functionInstance = Activator.CreateInstance(type) as ILearningRule;
                if (functionInstance == null)
                    throw new ApplicationException(string.Format(Resources.FoundFunctionNotIFunction, type.FullName));
                functionLists.Add(new FunctionListDescription { Name = functionInstance.Name, Type = type });
            }

            return functionLists;
        }

        private void UpdateCommands()
        {
            LoadTargetFile.RaiseCanExecuteChanged();
            LoadTrainingFile.RaiseCanExecuteChanged();
            TrainSingleStep.RaiseCanExecuteChanged();
            Train.RaiseCanExecuteChanged();
            ResetTraining.RaiseCanExecuteChanged();
        }
        #endregion

        #region Updates by other views and models
        private void OnNetworkChanged(INetwork network)
        {
            _model.Network = network;
        }

        private void ModelOnPropertyChanged(object sender, PropertyChangedEventArgs propertyChangedEventArgs)
        {
            if (propertyChangedEventArgs.PropertyName == PropertySupport.ExtractPropertyName(()=>_model.Error))
            {
                RaisePropertyChanged(()=>_model.Error);
            }
            else if (propertyChangedEventArgs.PropertyName == PropertySupport.ExtractPropertyName(()=> _model.Iteration))
            {
                RaisePropertyChanged(()=>_model.Iteration);
            }
        }

        void ModelCalculationCompletedEvent(object sender, EventArgs e)
        {
            RaisePropertyChanged(()=>_model.Error);
            RaisePropertyChanged(()=>_model.Iteration);
            _shellService.UpdateStatus(ShellStatus.Ready);
            UpdateCommands();
        }
        #endregion

        #region Event Commands

        public DelegateCommand<string> LoadTargetFile { get; private set; }
        public DelegateCommand<string> LoadTrainingFile { get; private set; }
        public DelegateCommand TrainSingleStep { get; private set; }
        public DelegateCommand Train { get; private set; }
        public DelegateCommand ResetTraining { get; private set; }

        private void OnLoadTargetFile(string targetFileName)
        {
            TargetFile = targetFileName;
            FileStream csvFile = new FileStream(targetFileName,FileMode.Open,FileAccess.Read,FileShare.Read);
            _model.Network.Target = new CsvStreamInput(csvFile);
        }

        private void OnLoadTrainingFile(string trainingFileName)
        {
            TrainingFile = trainingFileName;
            FileStream csvFile = new FileStream(trainingFileName,FileMode.Open,FileAccess.Read,FileShare.Read);
            _model.Network.Input = new CsvStreamInput(csvFile);
        }

        private void OnTraining()
        {
            if (_model.WorkInProgress)
                return;

            SetupNetwork();
            _model.Train();
            UpdateCommands();
            _shellService.UpdateStatus(ShellStatus.Processing);
        }

        private void OnSingleTraining()
        {
            /*SetupNetwork();
            

            _model.Network.Calculate();
            _model.Network.Learn();
            Error = _model.Network.ErrorFunction.Calculate();
            _iteration++;
            RaisePropertyChanged(() => Iteration);
            RaisePropertyChanged(() => Error);*/
        }

        private void OnResetTraining()
        {
            _model.CancelWork();
        }
        #endregion

        #region Properties
        public bool WorkInProgress
        {
            get { return _model.WorkInProgress; }
        }

        public int Iteration
        {
            get { return _model.Iteration; }
        }

        public double Error
        {
            get { return _model.Error; }
        }

        public double MaxError
        {
            get { return _model.MaxError; }
            set
            {
                _model.MaxError = value;
                RaisePropertyChanged(() => MaxError);
            }
        }

        public double Convergence 
        { 
            get { return _model.Convergence; } 
            set 
            { 
                _model.Convergence = value;
                RaisePropertyChanged(() => Convergence);
            }
        }

        public int ConvergenceDecayCount { get; set; }

        public double LearningRate
        {
            get { return _learningRate; }
            set
            {
                _learningRate = value;
                RaisePropertyChanged(() => LearningRate);
            }
        }

        public string TargetFile
        {
            get { return _targetFile; }
            set
            {
                if (_targetFile != value)
                {
                    _targetFile = value;
                    RaisePropertyChanged(() => TargetFile);
                }
                
            }
        }

        public string TrainingFile
        {
            get { return _trainingFile; }
            set
            {
                if (_trainingFile != value)
                {
                    _trainingFile = value;
                    RaisePropertyChanged(() => TrainingFile);
                }
            }
        }

        public IList<FunctionListDescription> ErrorFunctions
        {
            get { return _errorFunctions; }
        }

        public FunctionListDescription SelectedErrorFunction
        {
            get; set;
        }

        public IList<FunctionListDescription> LearningRules
        {
            get { return _learningRules; }
        }

        public FunctionListDescription SelectedLearningRule
        {
            get; set;
        }

        public IList<FunctionListDescription> ConvergenceFunctions
        {
            get { return _convergenceFunctions;  }
        }

        public FunctionListDescription SelectedConvergenceFunction
        {
            get; set;
        }
        #endregion

        #region Private Methods
        

        /// <summary>
        /// Setup the network with the data entered in the mask
        /// </summary>
        private void SetupNetwork()
        {
            if (WorkInProgress) return;

            _model.Network.ErrorFunction = Activator.CreateInstance(SelectedErrorFunction.Type) as IErrorFunction;
            if (_model.Network.ErrorFunction == null)
                throw new ApplicationException(String.Format(CultureInfo.InvariantCulture, Resources.LoadingErrorFunctionFailed,SelectedErrorFunction.Name));
            _model.Network.LearningRule = Activator.CreateInstance(SelectedLearningRule.Type) as ILearningRule;
            if (_model.Network.LearningRule == null)
                throw new ApplicationException(String.Format(CultureInfo.InvariantCulture, Resources.LoadingLearningRuleFailed, SelectedLearningRule.Name));
            _model.Network.ConvergenceSpeedFunction =
                Activator.CreateInstance(SelectedConvergenceFunction.Type, ConvergenceDecayCount) as
                IConvergenceSpeedFunction;
            if (_model.Network.ConvergenceSpeedFunction == null)
                throw new ApplicationException(String.Format(CultureInfo.InvariantCulture, Resources.LoadingConvergenceFunctionFailed, SelectedConvergenceFunction.Name));
            _model.Network.LearningRule.LearningRate = _learningRate;
            _model.Network.Input =
                new CsvStreamInput(new FileStream(TrainingFile, FileMode.Open, FileAccess.Read, FileShare.Read));
            _model.Network.Target =
                new CsvStreamInput(new FileStream(TargetFile, FileMode.Open, FileAccess.Read, FileShare.Read));
        }
        #endregion
    }
}
