﻿//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================

/* Additional portions copyright:
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using EnnfDesigner.Common.Properties;
using Microsoft.Practices.Prism;
using Microsoft.Practices.Prism.Modularity;

namespace EnnfDesigner.Common.ModuleCatalog
{
    /// <summary>
    /// This class serves as an extender of the Prism Module Catalog loader from a configuration file.
    /// The reason for this is that the config can now offer a lookup for what kind of network type
    /// it is configured
    /// </summary>
    /// <remarks>This code is taken from the original Prism 4.1 Library and modified to suit the Ennf needs</remarks>
    public class EnnfConfigurationModuleCatalog : Microsoft.Practices.Prism.Modularity.ModuleCatalog 
    {
        /// <summary>
        /// Builds an instance of ConfigurationModuleCatalog with a <see cref="ConfigurationStore"/> as the default store.
        /// </summary>
        public EnnfConfigurationModuleCatalog() 
        {
            Store = new ConfigurationStore();
        }

        /// <summary>
        /// Gets or sets the store where the configuration is kept.
        /// </summary>
        public IConfigurationStore Store { get; set; }

        /// <summary>
        /// Loads the catalog from the configuration.
        /// </summary>
        protected override void InnerLoad()
        {
            if (Store == null)
            {
                throw new InvalidOperationException(Resources.ConfigurationStoreCannotBeNull);
            }

            EnsureModulesDiscovered();
        }

        private static string GetFileAbsoluteUri(string filePath)
        {
            UriBuilder uriBuilder = new UriBuilder
                                        {
                                            Host = String.Empty,
                                            Scheme = Uri.UriSchemeFile,
                                            Path = Path.GetFullPath(filePath)
                                        };
            Uri fileUri = uriBuilder.Uri;

            return fileUri.ToString();
        }

        private void EnsureModulesDiscovered()
        {
            ModulesConfigurationSection section = Store.RetrieveModuleConfigurationSection();

            if (section == null) return;
            foreach (EnnfModuleConfigurationElement element in section.Modules)
            {
                IList<string> dependencies = new List<string>();

                if (element.Dependencies.Count > 0)
                {
                    foreach (ModuleDependencyConfigurationElement dependency in element.Dependencies)
                    {
                        dependencies.Add(dependency.ModuleName);
                    }
                }

                EnnfModuleInfo moduleInfo = new EnnfModuleInfo(element.ModuleName, element.ModuleType,element.NetworkType)
                                            {
                                                Ref = GetFileAbsoluteUri(element.AssemblyFile),
                                                InitializationMode = element.StartupLoaded ? InitializationMode.WhenAvailable : InitializationMode.OnDemand
                                            };
                moduleInfo.DependsOn.AddRange(dependencies.ToArray());
                AddModule(moduleInfo);
            }
        }
    }
}
