﻿//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================

/* Additional portions copyright:
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using Microsoft.Practices.Prism.Modularity;

namespace EnnfDesigner.Common.ModuleCatalog
{
    /// <summary>
    /// A collection of <see cref="EnnfModuleConfigurationElement"/>.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1010:CollectionsShouldImplementGenericInterface")]
    public class EnnfModuleConfigurationElementCollection : ModuleConfigurationElementCollection
    {
        /// <summary>
        /// Initializes a new instance of <see cref="EnnfModuleConfigurationElementCollection"/>.
        /// </summary>
        public EnnfModuleConfigurationElementCollection()
        {
        }

        /// <summary>
        /// Initializes a new <see cref="EnnfModuleConfigurationElementCollection"/>.
        /// </summary>
        /// <param name="modules">The initial set of <see cref="EnnfModuleConfigurationElement"/>.</param>
        /// <exception cref="ArgumentNullException">An <see cref="ArgumentNullException"/> is thrown if <paramref name="modules"/> is <see langword="null"/>.</exception>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2214:DoNotCallOverridableMethodsInConstructors")]
        public EnnfModuleConfigurationElementCollection(IEnumerable<EnnfModuleConfigurationElement> modules)
        {
            if (modules == null) throw new ArgumentNullException("modules");
            foreach (EnnfModuleConfigurationElement module in modules)
            {
                BaseAdd(module);
            }
        }

        /// <summary>
        /// Gets a value indicating whether an exception should be raised if a duplicate element is found.
        /// This property will always return true.
        /// </summary>
        /// <value>A <see cref="bool"/> value.</value>
        protected override bool ThrowOnDuplicate
        {
            get { return true; }
        }

        ///<summary>
        ///Gets the type of the <see cref="T:System.Configuration.ConfigurationElementCollection" />.
        ///</summary>
        ///<value>
        ///The <see cref="T:System.Configuration.ConfigurationElementCollectionType" /> of this collection.
        ///</value>
        public override ConfigurationElementCollectionType CollectionType
        {
            get { return ConfigurationElementCollectionType.BasicMap; }
        }

        ///<summary>
        ///Gets the name used to identify this collection of elements in the configuration file when overridden in a derived class.
        ///</summary>
        ///<value>
        ///The name of the collection; otherwise, an empty string.
        ///</value>
        protected override string ElementName
        {
            get { return "module"; }
        }

        /// <summary>
        /// Gets the <see cref="EnnfModuleConfigurationElement"/> located at the specified index in the collection.
        /// </summary>
        /// <param name="index">The index of the element in the collection.</param>
        /// <returns>A <see cref="EnnfModuleConfigurationElement"/>.</returns>
        public new EnnfModuleConfigurationElement this[int index]
        {
            get { return (EnnfModuleConfigurationElement)BaseGet(index); }
        }

        /// <summary>
        /// Adds a <see cref="EnnfModuleConfigurationElement"/> to the collection.
        /// </summary>
        /// <param name="module">A <see cref="EnnfModuleConfigurationElement"/> instance.</param>
        public void Add(EnnfModuleConfigurationElement module)
        {
            BaseAdd(module);
        }

        /// <summary>
        /// Searches the collection for all the <see cref="EnnfModuleConfigurationElement"/> that match the specified predicate.
        /// </summary>
        /// <param name="match">A <see cref="Predicate{T}"/> that implements the match test.</param>
        /// <returns>A <see cref="List{T}"/> with the successful matches.</returns>
        /// <exception cref="ArgumentNullException">An <see cref="ArgumentNullException"/> is thrown if <paramref name="match"/> is null.</exception>
        public IList<EnnfModuleConfigurationElement> FindAll(Predicate<EnnfModuleConfigurationElement> match)
        {
            if (match == null) throw new ArgumentNullException("match");

            return this.Cast<EnnfModuleConfigurationElement>().Where(moduleElement => match(moduleElement)).ToList();
        }

        /// <summary>
        /// Creates a new <see cref="EnnfModuleConfigurationElement"/>.
        /// </summary>
        /// <returns>A <see cref="EnnfModuleConfigurationElement"/>.</returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new EnnfModuleConfigurationElement();
        }

        /// <summary>
        /// Gets the element key for a specified configuration element when overridden in a derived class.
        /// </summary>
        /// <param name="element">The <see cref="T:System.Configuration.ConfigurationElement" /> to return the key for. </param>
        /// <returns>
        /// An <see cref="T:System.Object" /> that acts as the key for the specified <see cref="T:System.Configuration.ConfigurationElement" />.
        /// </returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((EnnfModuleConfigurationElement)element).ModuleName;
        }
    }
}
