﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Data;
using Ennf;
using EnnfDesigner.Common.Events;
using EnnfDesigner.Common.ModuleCatalog;
using EnnfDesigner.Common.RegionControls;
using EnnfDesigner.Properties;
using EnnfDesigner.Services.Interface;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Prism.ViewModel;
using EnnfDesigner.Models;
using Microsoft.Practices.Unity;

namespace EnnfDesigner.ViewModels
{
    public class ShellViewModel : NotificationObject
    {
        private readonly NetworkModel _model;
        private readonly IRegionManager _regionManager;
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private string _networkPath;
        private ICollectionView _networks;

        public ShellViewModel(IRegionManager regionManager, IEventAggregator eventAggregator, IUnityContainer container)
        {
            // get the underlying Model and assign Infrastructure
            _container = container;
            _model = _container.Resolve<NetworkModel>();
            _eventAggregator = eventAggregator;
            _regionManager = regionManager;
            // Setup Commands
            LoadNetwork = new CompositeCommand();
            LoadNetwork.RegisterCommand(new DelegateCommand(OnSetNetworkFilePath));
            LoadNetwork.RegisterCommand(new DelegateCommand(OnLoadNetwork, () => !_model.Loaded));
            CloseNetwork = new DelegateCommand(OnCloseNetwork, () => _model.Loaded);
            CloseApplication = new DelegateCommand(OnCloseApplication);
            OpenConsole = new DelegateCommand<INetwork>(OnOpenConsole);
            // Initialize the data to current state
            _model.PropertyChanged += OnPropertyChanged;
            if (_model.Loaded)
            {
                Networks = new CollectionView(_model.Networks);
            }
        }

        /// <summary>
        /// This method is called when one of the properties change on the Model. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="propertyChangedEventArgs"></param>
        private void OnPropertyChanged(object sender, PropertyChangedEventArgs propertyChangedEventArgs)
        {
            if (propertyChangedEventArgs.PropertyName == PropertySupport.ExtractPropertyName(() => _model.Loaded))
            {
                foreach (DelegateCommand command in LoadNetwork.RegisteredCommands)
                {
                    command.RaiseCanExecuteChanged();
                }
                CloseNetwork.RaiseCanExecuteChanged();
                CloseApplication.RaiseCanExecuteChanged();
                Networks = _model.Networks == null ? null : new CollectionView(_model.Networks);
            }
        }

        private void ContinueUnloading(NavigationResult result)
        {
            if (result.Result == false)
                return;

            _model.CloseNetwork();
        }

        #region Properties
        public ICollectionView Networks
        {
            get
            {
                return _networks;
            }
            private set 
            { 
                _networks = value;
                RaisePropertyChanged(() => Networks);
            }
        }
        public bool NetworkLoaded
        {
            get { return _model.Loaded; }
        }

        public string NetworkFilePath
        {
            get
            {
                return _networkPath;
            } 
            set
            {
                _networkPath = value; 
                RaisePropertyChanged(() => NetworkFilePath);
            }
        }
        #endregion

        #region Commands

        public CompositeCommand LoadNetwork { get; private set; }
        public DelegateCommand CloseNetwork { get; private set; }
        public DelegateCommand CloseApplication { get; private set; }
        public DelegateCommand<INetwork> OpenConsole { get; private set; }

        private void OnLoadNetwork()
        {
            if (string.IsNullOrEmpty(NetworkFilePath))
                return; 

            _model.LoadNetwork(NetworkFilePath);
            Networks = new CollectionView(_model.Networks);

            _regionManager.RequestNavigate(RegionNames.LeftRegion,
                                           new Uri(NavigationNames.NetworkExplorerName, UriKind.Relative));
           
        }

        private void OnSetNetworkFilePath()
        {
            IFileService service = _container.Resolve<IFileService>();
            NetworkFilePath = service.GetFilePath(Resources.EnnfFileTypeExtension, Resources.EnnfFileTypeFilter);
        }
     
        private void OnCloseNetwork()
        {
            _model.CloseNetwork();
            _regionManager.RequestNavigate(RegionNames.MainRegion, new Uri(NavigationNames.EmptyMain, UriKind.Relative), ContinueUnloading);
        }

        private void OnOpenConsole(INetwork network)
        {
            EnnfModuleManager moduleManager = _container.Resolve<IModuleManager>() as EnnfModuleManager;
            if (moduleManager == null)
                throw new ApplicationException(Resources.IncorrectModuleManagerType);

            // TODO: change as we probably don't want modeltype but network-type for modules 
            moduleManager.EnsureModule(_model.ModelType);
            _regionManager.RequestNavigate(RegionNames.MainRegion,
                                           new Uri(NavigationNames.NetworkConsole, UriKind.Relative));
            NetworkSelectedEvent selectedEvent = _eventAggregator.GetEvent<NetworkSelectedEvent>();
            if (selectedEvent != null)
                selectedEvent.Publish(network);

        }
        
        private void OnCloseApplication()
        {
            if (Application.Current != null)
                Application.Current.Shutdown();
        }
        #endregion
    }
}
