﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System.Data.Entity;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Ennf.DbEntityAdapter;
using Ennf.Functions.ActivationFunctions;
using Ennf.Functions.ConvergenceSpeedFunctions;
using Ennf.Functions.ErrorFunctions;
using Ennf.Functions.InitializationFunctions;
using Ennf.LearningRules.SupervisedLearning.ErrorBackpropagation;
using Ennf.NetworkModels.BasicFeedForward;
using EnnfTest.Properties;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Ennf.Infrastructure;
using Microsoft.Practices.Unity;

namespace EnnfTest.NetworkModelsFixture
{
    /// <summary>
    /// This class simulates the underlying database layer for unit testing
    /// </summary>
    public class BasicFeedForwardPersistenceModelFixture : IBasicFeedForwardPersistenceModel
    {
        #region DataModel enum

        public enum DataModel
        {
            Standard,
            PlusMinus,
            PlusMinusDefinedWeights
        }

        #endregion
        
        private DbSetFixture<BasicFeedForwardConnection> _connectionSet;
        private DbSetFixture<BasicFeedForwardNetwork> _networkSet;
        private DbSetFixture<BasicFeedForwardNode> _nodeSet;

        public BasicFeedForwardPersistenceModelFixture(IUnityContainer container)
        {
            UnityContainer = container;
            Logger = container.Resolve<LogWriter>();
            // Create the basic network
            ResetData();
        }

        public DbSetFixture<BasicFeedForwardNetwork> NetworkDbSet
        {
            get { return _networkSet; }
        }

        public DbSetFixture<BasicFeedForwardNode> NodeDbSet
        {
            get { return _nodeSet; }
        }

        public DbSetFixture<BasicFeedForwardConnection> ConnectionDbSet
        {
            get { return _connectionSet; }
        }

        #region Interface Implementations
        [Dependency]
        public LogWriter Logger { get; set; }
        [Dependency]
        public IUnityContainer UnityContainer { get; set; }

        public IDbSet<BasicFeedForwardNetwork> NetworkSet
        {
            get { return _networkSet; }
            set { _networkSet = value as DbSetFixture<BasicFeedForwardNetwork>; }
        }

        public IDbSet<BasicFeedForwardNode> NodeSet
        {
            get { return _nodeSet; }
            set { _nodeSet = value as DbSetFixture<BasicFeedForwardNode>; }
        }

        public IDbSet<BasicFeedForwardConnection> ConnectionSet
        {
            get { return _connectionSet; }
            set { _connectionSet = value as DbSetFixture<BasicFeedForwardConnection>; }
        }

        public IDbEntityEntry<TEntity> Entry<TEntity>(TEntity entity) where TEntity : class
        {
            return new DbEntityEntryTestFixture<TEntity>(entity);
        }

        public int SaveChanges()
        {
            // TODO: count the amount of changes were stored away
            return 0;
        }

        public void Close()
        {
            throw new System.NotImplementedException();
        }

        #endregion

        private byte[] GetSerializedObject(object toserialize)
        {
            var stream = new MemoryStream();
            var formatter = new BinaryFormatter();

            formatter.Serialize(stream, toserialize);
            byte[] toreturn = stream.ToArray();

            return toreturn;
        }

        public void ResetData()
        {
            ResetData(DataModel.Standard);
        }

        public void ResetData(DataModel model)
        {
            switch (model)
            {
                case DataModel.Standard:
                    DataModelStandard();
                    break;
                case DataModel.PlusMinus:
                    DataModelPlusMinus(false);
                    break;
                case DataModel.PlusMinusDefinedWeights:
                    DataModelPlusMinus(true);
                    break;
            }
        }

        private void DataModelStandard()
        {
            _networkSet = new DbSetFixture<BasicFeedForwardNetwork>();
            _nodeSet = new DbSetFixture<BasicFeedForwardNode>();
            _connectionSet = new DbSetFixture<BasicFeedForwardConnection>();
            // Setup the Model now
            BasicFeedForwardNetwork network = new BasicFeedForwardNetwork
                              {
                                  Identifier = 1,
                                  InputObject = GetSerializedObject(new TestInput()),
                                  OutputObject = GetSerializedObject(new TestOutput()),
                                  
                              };
            UnityContainer.BuildUp(network);
            network.InitializationFunction = new RandomUnitFunction(-1,1);
            
            _networkSet.Add(network);
            BasicFeedForwardNetwork network2 = new BasicFeedForwardNetwork { Identifier = 2 };
            _networkSet.Add(network2);
            // Nodes
            BasicFeedForwardNode node = new BasicFeedForwardNode(network)
                           {NetworkIdentifierFK = 1, NodeId = 1, AddressX = 0, AddressY = 0, AddressZ = 0};
            UnityContainer.BuildUp(node);
            _nodeSet.Add(node);
            // Connections
            BasicFeedForwardConnection conn = new BasicFeedForwardConnection(network)
                           {
                               InputNode = node,
                               OutputNode = node
                           };
            UnityContainer.BuildUp(conn);
            _connectionSet.Add(conn);
            BasicFeedForwardInputConnection conn2 = new BasicFeedForwardInputConnection(network)
                                                        {
                                                            Input_NodeId_FK = -1, 
                                                            OutputNode = node
                                                        };
            UnityContainer.BuildUp(conn2);
            _connectionSet.Add(conn2);
            BasicFeedForwardOutputConnection conn3 = new BasicFeedForwardOutputConnection(network)
                                                         {
                                                             Output_NodeId_FK = -1, 
                                                             InputNode = node
                                                         };
            UnityContainer.BuildUp(conn3);
            _connectionSet.Add(conn3);

            _networkSet.ResetCount();
            _nodeSet.ResetCount();
            _connectionSet.ResetCount();
        }

        private void DataModelPlusMinus(bool definedWeight)
        {
            _networkSet = new DbSetFixture<BasicFeedForwardNetwork>();
            _nodeSet = new DbSetFixture<BasicFeedForwardNode>();
            _connectionSet = new DbSetFixture<BasicFeedForwardConnection>();

            // create the network
            BasicFeedForwardNetwork network = new BasicFeedForwardNetwork
                                   {
                                       Identifier = 1,
                                       InputObject =
                                           GetSerializedObject(
                                               new CsvStreamInput(new FileStream(Settings.Default.CsvInputName,
                                                                                 FileMode.Open))),
                                        OutputObject = GetSerializedObject(new CsvStreamOutput())
                                   };
            UnityContainer.BuildUp(network);
            network.LearningRule = new ErrorBackpropagationRule {LearningRate = 0.2};
            network.ConvergenceSpeedFunction = new MovingAverageConvergenceSpeed(10);
            network.Target = new CsvStreamInput(new FileStream(Settings.Default.CsvTargetName, FileMode.Open));
            network.ErrorFunction = new SumOfSquaresFunction();
            network.InitializationFunction = new RandomUnitFunction(-1, 1);
            _networkSet.Add(network);
            // create the 9 nodes that take the input
            BasicFeedForwardNode inputnode11 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 1,
                                                    AddressX = 0,
                                                    AddressY = 0,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode11);
            _nodeSet.Add(inputnode11);
            BasicFeedForwardNode inputnode12 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 2,
                                                    AddressX = 0,
                                                    AddressY = 1,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode12);
            _nodeSet.Add(inputnode12);
            BasicFeedForwardNode inputnode13 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 3,
                                                    AddressX = 0,
                                                    AddressY = 2,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode13);
            _nodeSet.Add(inputnode13);
            BasicFeedForwardNode inputnode21 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 4,
                                                    AddressX = 1,
                                                    AddressY = 0,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode21);
            _nodeSet.Add(inputnode21);
            BasicFeedForwardNode inputnode22 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 5,
                                                    AddressX = 1,
                                                    AddressY = 1,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode22);
            _nodeSet.Add(inputnode22);
            BasicFeedForwardNode inputnode23 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 6,
                                                    AddressX = 1,
                                                    AddressY = 2,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode23);
            _nodeSet.Add(inputnode23);
            BasicFeedForwardNode inputnode31 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 7,
                                                    AddressX = 2,
                                                    AddressY = 0,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode31);
            _nodeSet.Add(inputnode31);
            BasicFeedForwardNode inputnode32 = new BasicFeedForwardNode(network)
                                {
                                    NetworkIdentifierFK = network.Identifier,
                                    NodeId = 8,
                                    AddressX = 2,
                                    AddressY = 1,
                                    AddressZ = 0
                                };
            UnityContainer.BuildUp(inputnode32);
            _nodeSet.Add(inputnode32);
            BasicFeedForwardNode inputnode33 = new BasicFeedForwardNode(network)
                                                {
                                                    NetworkIdentifierFK = network.Identifier,
                                                    NodeId = 9,
                                                    AddressX = 2,
                                                    AddressY = 2,
                                                    AddressZ = 0
                                                };
            UnityContainer.BuildUp(inputnode33);
            _nodeSet.Add(inputnode33);
            // create the output node
            BasicFeedForwardNode outputnode1 = new BasicFeedForwardNode(network)
                                                  {
                                                      NetworkIdentifierFK = network.Identifier,
                                                      NodeId = 10,
                                                      AddressX = 0,
                                                      AddressY = 0,
                                                      AddressZ = 1
                                                  };
            UnityContainer.BuildUp(outputnode1);
            outputnode1.ActivationFunction = new LogisticSigmoidFunction();
            _nodeSet.Add(outputnode1);
            BasicFeedForwardNode outputnode2 = new BasicFeedForwardNode(network)
                                                   {
                                                       NetworkIdentifierFK = network.Identifier,
                                                       NodeId = 11,
                                                       AddressX = 1,
                                                       AddressY = 0,
                                                       AddressZ = 1
                                                   };
            UnityContainer.BuildUp(outputnode2);
            outputnode2.ActivationFunction = new LogisticSigmoidFunction();
            _nodeSet.Add(outputnode2);
            // create the input connections
            BasicFeedForwardInputConnection connection1 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 1,
                                                                  OutputNode = inputnode11
                                                              };
            UnityContainer.BuildUp(connection1);
            _connectionSet.Add(connection1);
            BasicFeedForwardInputConnection connection2 = new BasicFeedForwardInputConnection(network)
                                                              { 
                                                                  ConnectionId = 2,
                                                                  OutputNode = inputnode12
                                                              };
            UnityContainer.BuildUp(connection2);
            _connectionSet.Add(connection2);
            BasicFeedForwardInputConnection connection3 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 3,
                                                                  OutputNode = inputnode13
                                                              };
            UnityContainer.BuildUp(connection3);
            _connectionSet.Add(connection3);
            BasicFeedForwardInputConnection connection4 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 4,
                                                                  OutputNode = inputnode21
                                                              };
            UnityContainer.BuildUp(connection4);
            _connectionSet.Add(connection4);
            BasicFeedForwardInputConnection connection5 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 5,
                                                                  OutputNode = inputnode22
                                                              };
            UnityContainer.BuildUp(connection5);
            _connectionSet.Add(connection5);
            BasicFeedForwardInputConnection connection6 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 6,
                                                                  OutputNode = inputnode23
                                                              };
            UnityContainer.BuildUp(connection6);
            _connectionSet.Add(connection6);
            BasicFeedForwardInputConnection connection7 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 7,
                                                                  OutputNode = inputnode31
                                                              };
            UnityContainer.BuildUp(connection7);
            _connectionSet.Add(connection7);
            BasicFeedForwardInputConnection connection8 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 8,
                                                                  OutputNode = inputnode32
                                                              };
            UnityContainer.BuildUp(connection8);
            _connectionSet.Add(connection8);
            BasicFeedForwardInputConnection connection9 = new BasicFeedForwardInputConnection(network)
                                                              {
                                                                  ConnectionId = 9,
                                                                  OutputNode = inputnode33
                                                              };
            UnityContainer.BuildUp(connection9);
            _connectionSet.Add(connection9);
            // connect the input to the output 1
            BasicFeedForwardConnection connection11 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 11,
                                                                InputNode = inputnode11,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection11);
            if (definedWeight)
                connection11.Weight = 0.73653665824632009;
            _connectionSet.Add(connection11);

            BasicFeedForwardConnection connection12 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 12,
                                                                InputNode = inputnode12,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection12);
            if (definedWeight)
                connection12.Weight = 0.063936659164697235;
            _connectionSet.Add(connection12);

            BasicFeedForwardConnection connection13 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 13,
                                                                InputNode = inputnode13,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection13);
            if (definedWeight)
                connection13.Weight = 0.8115699881741637;
            _connectionSet.Add(connection13);

            BasicFeedForwardConnection connection14 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 14,
                                                                InputNode = inputnode21,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection14);
            if (definedWeight)
                connection14.Weight = 0.0046137054472294194;
            _connectionSet.Add(connection14);

            BasicFeedForwardConnection connection15 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 15,
                                                                InputNode = inputnode22,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection15);
            if (definedWeight)
                connection15.Weight = 0.050784169719919642;
            _connectionSet.Add(connection15);

            BasicFeedForwardConnection connection16 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 16,
                                                                InputNode = inputnode23,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection16);
            if (definedWeight)
                connection16.Weight = 0.2021876378926391;
            _connectionSet.Add(connection16);

            BasicFeedForwardConnection connection17 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 17,
                                                                InputNode = inputnode31,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection17);
            if (definedWeight)
                connection17.Weight = 0.78405931675064344;
            _connectionSet.Add(connection17);

            BasicFeedForwardConnection connection18 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 18,
                                                                InputNode = inputnode32,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection18);
            if (definedWeight)
                connection18.Weight = 0.08709968956517973;
            _connectionSet.Add(connection18);

            BasicFeedForwardConnection connection19 = new BasicFeedForwardConnection(network)
                                                            {
                                                                ConnectionId = 19,
                                                                InputNode = inputnode33,
                                                                OutputNode = outputnode1
                                                            };
            UnityContainer.BuildUp(connection19);
            if (definedWeight)
                connection19.Weight = 0.081216060128629239;
            _connectionSet.Add(connection19);

            // connect the input to the output 2
            BasicFeedForwardConnection connection21 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 21,
                                                              InputNode = inputnode11,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection21);
            if (definedWeight)
                connection21.Weight = 0.52111643157951371;
            _connectionSet.Add(connection21);

            BasicFeedForwardConnection connection22 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 22,
                                                              InputNode = inputnode12,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection22);
            if (definedWeight)
                connection22.Weight = 0.71356512639371916;
            _connectionSet.Add(connection22);

            BasicFeedForwardConnection connection23 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 23,
                                                              InputNode = inputnode13,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection23);
            if (definedWeight)
                connection23.Weight = 0.18438530861604274;
            _connectionSet.Add(connection23);

            BasicFeedForwardConnection connection24 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 24,
                                                              InputNode = inputnode21,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection24);
            if (definedWeight)
                connection24.Weight = 0.55588729565771633;
            _connectionSet.Add(connection24);

            BasicFeedForwardConnection connection25 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 25,
                                                              InputNode = inputnode22,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection25);
            if (definedWeight)
                connection25.Weight = 0.68771520382152651;
            _connectionSet.Add(connection25);

            BasicFeedForwardConnection connection26 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 26,
                                                              InputNode = inputnode23,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection26);
            if (definedWeight)
                connection26.Weight = 0.65927010246518536;
            _connectionSet.Add(connection26);

            BasicFeedForwardConnection connection27 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 27,
                                                              InputNode = inputnode31,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection27);
            if (definedWeight)
                connection27.Weight = 0.17694662798985217;
            _connectionSet.Add(connection27);

            BasicFeedForwardConnection connection28 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 28,
                                                              InputNode = inputnode32,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection28);
            if (definedWeight)
                connection28.Weight = 0.24595051596218279;
            _connectionSet.Add(connection28);

            BasicFeedForwardConnection connection29 = new BasicFeedForwardConnection(network)
                                                          {
                                                              ConnectionId = 29,
                                                              InputNode = inputnode33,
                                                              OutputNode = outputnode2
                                                          };
            UnityContainer.BuildUp(connection29);
            if (definedWeight)
                connection29.Weight = 0.29816400506448187;
            _connectionSet.Add(connection29);

            // connect the output node to the 
            BasicFeedForwardOutputConnection finalConnection1 = new BasicFeedForwardOutputConnection(network)
                                                                {
                                                                    ConnectionId = 30,
                                                                    InputNode = outputnode1
                                                                };
            UnityContainer.BuildUp(finalConnection1);
            _connectionSet.Add(finalConnection1);
            BasicFeedForwardOutputConnection finalConnection2 = new BasicFeedForwardOutputConnection(network)
                                                                {
                                                                    ConnectionId = 31,
                                                                    InputNode = outputnode2
                                                                };
            UnityContainer.BuildUp(finalConnection2);
            _connectionSet.Add(finalConnection2);

            _networkSet.ResetCount();
            _nodeSet.ResetCount();
            _connectionSet.ResetCount();
        }

        public void LoadFile(string filename)
        {
        }

        public void ConnectDatabase(string connectionString)
        {
            
        }
    }
}