﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.IO;
using Ennf;
using Ennf.Functions.ConvergenceSpeedFunctions;
using Ennf.Functions.ErrorFunctions;
using Ennf.Functions.InitializationFunctions;
using Ennf.Infrastructure;
using Ennf.LearningRules.SupervisedLearning.ErrorBackpropagation;
using Ennf.Utilities;
using EnnfTest.NetworkModelsFixture;
using EnnfTest.Properties;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace EnnfTest.UnitTests
{
    [TestClass]
    public class BasicCalculationOperations
    {
        private static Container _ennfContainer;
        private static IUnityContainer _container;
        private static BasicFeedForwardPersistenceModelFixture _persistenceModel;

        [ClassInitialize]
        public static void MyClassInitialize(TestContext testContext)
        {
            _container = new UnityContainer();
            _container.LoadConfiguration(Settings.Default.UnityContainerName);

            _ennfContainer = new Container(_container);
            // It really doesn't matter what we try to load here, we just need to initialize
            // the network model and the rest will be taken care by the injected Model fixtre
            _ennfContainer.LoadFile(Resources.TestXmlFile);
            _persistenceModel = _ennfContainer.NetworkModel.PersistenceModel as BasicFeedForwardPersistenceModelFixture;
            Assert.IsNotNull(_persistenceModel);
        }

        [TestInitialize]
        public void SetupData()
        {
            _persistenceModel.ResetData(BasicFeedForwardPersistenceModelFixture.DataModel.PlusMinus);
        }

        [TestCategory("UnitTests"), TestCategory("Calculation Operations Tests"), TestMethod]
        public void TestCalculationValidation()
        {
            _persistenceModel.ResetData(BasicFeedForwardPersistenceModelFixture.DataModel.PlusMinusDefinedWeights);
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            // Calculate the input with the specific weights
            network.Calculate();
            CsvStreamOutput output = network.Output as CsvStreamOutput;
            Assert.IsNotNull(output);
            Assert.AreEqual(0.6007574, Math.Round(output.Outputs[0][0], 7));
            Assert.AreEqual(0.9459555, Math.Round(output.Outputs[0][1], 7));
            
        }

        [TestCategory("UnitTests"), TestCategory("Calculation Operations Tests"), TestMethod]
        public void TestCalculationAndErrorValidation()
        {
            _persistenceModel.ResetData(BasicFeedForwardPersistenceModelFixture.DataModel.PlusMinusDefinedWeights);
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            network.Calculate();
            // now do the learning and validate the Error Function
            network.Learn();
            Assert.AreEqual(0.5271133, Math.Round(network.ErrorFunction.Calculate(),7));
        }

        [TestCategory("UnitTests"), TestCategory("Calculation Operations Tests"), TestMethod]
        public void TestCalculationAndConvergenceAndLearningIncremental()
        {
            _persistenceModel.ResetData(BasicFeedForwardPersistenceModelFixture.DataModel.PlusMinus);
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            double errorValue = 1.0;
            double convergence = 1.0;
            int counter = 0;

            while (convergence > 0.000001 && errorValue > 0.01)
            {
                network.Calculate();
                network.Learn();
                errorValue = network.ErrorFunction.Calculate();
                convergence = network.ConvergenceSpeedFunction.Calculate();
                counter++;
            }
            Assert.IsTrue(counter < 2000);
        }

        [TestCategory("UnitTests"), TestCategory("Calculation Operations Tests"), TestMethod]
        public void UseLayerHelperAndTestCalculation()
        {
            INetwork network = _ennfContainer.NetworkModel.CreateNetwork();
            network.Input = new CsvStreamInput(new FileStream(Settings.Default.CsvInputName, FileMode.Open));
            network.Output = new CsvStreamOutput();
            network.Target = new CsvStreamInput(new FileStream(Settings.Default.CsvTargetName, FileMode.Open));
            network.LearningRule = new ErrorBackpropagationRule {LearningRate = 0.1};
            network.ErrorFunction = new SumOfSquaresFunction();
            network.ConvergenceSpeedFunction = new MovingAverageConvergenceSpeed(10);
            network.InitializationFunction = new RandomUnitFunction(-1,1);

            LayerHelper helper = new LayerHelper(network);
            helper.CreateInputs(9);
            helper.CreateLayer(5);
            helper.CreateLayer(2);
            helper.ConnectLayerToInput(0);
            helper.ConnectLayerToLayer(0,1);
            helper.CreateOutput(1);

            double errorValue = 1.0;
            double convergence = 1.0;
            double lastConvergence = 1.5;
            int counter = 0;

            while (convergence > 0.001 && errorValue > 0.01 && convergence < lastConvergence)
            {
                network.Calculate();
                network.Learn();
                errorValue = network.ErrorFunction.Calculate();
                lastConvergence = convergence;
                convergence = network.ConvergenceSpeedFunction.Calculate();
                counter++;
            }

            Assert.IsTrue(counter > 1);
        }
    }
}