﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System.Linq;
using Ennf;
using EnnfTest.NetworkModelsFixture;
using EnnfTest.Properties;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace EnnfTest.UnitTests
{
    /// <summary>
    /// Summary description for BasicConnectionOperations
    /// </summary>
    [TestClass]
    public class BasicConnectionOperations
    {
        private static Container _ennfContainer;
        private static IUnityContainer _container;
        private static BasicFeedForwardPersistenceModelFixture _persistenceModel;

        [ClassInitialize]
        public static void MyClassInitialize(TestContext testContext)
        {
            _container = new UnityContainer();
            _container.LoadConfiguration(Settings.Default.UnityContainerName);

            _ennfContainer = new Container(_container);
            // It really doesn't matter what we try to load here, we just need to initialize
            // the network model and the rest will be taken care by the injected Model fixtre
            _ennfContainer.LoadFile(Resources.TestXmlFile);
            _persistenceModel = _ennfContainer.NetworkModel.PersistenceModel as BasicFeedForwardPersistenceModelFixture;
            Assert.IsNotNull(_persistenceModel);
        }

        [TestInitialize]
        public void SetupData()
        {
            _persistenceModel.ResetData();
        }

        #region General Connections

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void AddConnection()
        {
            // Execute data
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IConnection newCon = network.CreateConnection();
            INode node = network.Nodes.First();
            newCon.InputNode = node;
            newCon.OutputNode = node;
            network.Connections.Add(newCon);

            _ennfContainer.NetworkModel.Save();

            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.AddCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void RemoveConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IConnection newCon = network.Connections.First();
            network.Connections.Remove(newCon);

            _ennfContainer.NetworkModel.Save();

            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.RemoveCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void UpdateConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IConnection newCon = network.Connections.First();
            INode node = network.CreateNode();
            network.Nodes.Add(node);
            newCon.InputNode = node;
            newCon.OutputNode = node;

            _ennfContainer.NetworkModel.Save();
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void ReadConnectionNodes()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IConnection newCon = network.Connections.First();
            INode node = newCon.InputNode;
            Assert.IsNotNull(node);
            node = newCon.OutputNode;

            Assert.IsNotNull(node);
        }

        #endregion

        #region InputConnections

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void AddInputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInputConnection newCon = network.CreateInputConnection();

            INode node = network.Nodes.Last();
            newCon.OutputNode = node;
            network.InputConnections.Add(newCon);

            _ennfContainer.NetworkModel.Save();
            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.AddCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void ReadInputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInputConnection newCon = network.InputConnections.First();
            INode node = newCon.OutputNode;
            IInput input = newCon.Input;

            Assert.IsNotNull(node);
            Assert.IsNotNull(input);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void RemoveInputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInputConnection newCon = network.InputConnections.First();

            network.InputConnections.Remove(newCon);

            _ennfContainer.NetworkModel.Save();
            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.RemoveCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void UpdateInputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInputConnection newCon = network.InputConnections.First();
            INode newNode = network.CreateNode();
            network.Nodes.Add(newNode);
            newCon.OutputNode = newNode;

            _ennfContainer.NetworkModel.Save();
        }

        #endregion

        #region OutputConnections

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void AddOutputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutputConnection newCon = network.CreateOutputConnection();

            INode node = network.Nodes.Last();
            newCon.InputNode = node;
            network.OutputConnections.Add(newCon);

            _ennfContainer.NetworkModel.Save();
            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.AddCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void ReadOutputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutputConnection newCon = network.OutputConnections.First();
            INode node = newCon.InputNode;
            IOutput input = newCon.Output;

            Assert.IsNotNull(node);
            Assert.IsNotNull(input);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void RemoveOutputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutputConnection newCon = network.OutputConnections.First();

            network.OutputConnections.Remove(newCon);

            _ennfContainer.NetworkModel.Save();
            Assert.AreEqual(1, _persistenceModel.ConnectionDbSet.RemoveCount);
        }

        [TestCategory("UnitTests"), TestCategory("Test Connection Operations"), TestMethod]
        public void UpdateOutputConnection()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutputConnection newCon = network.OutputConnections.First();
            INode newNode = network.CreateNode();
            network.Nodes.Add(newNode);
            newCon.InputNode = newNode;

            _ennfContainer.NetworkModel.Save();
        }

        #endregion
    }
}