﻿/*
 * ennf - Eddie's Neural Network Framework
 * Copyright 2011 Eduard Ralph
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Linq;
using Ennf;
using EnnfTest.NetworkModelsFixture;
using EnnfTest.Properties;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace EnnfTest.UnitTests
{
    /// <summary>
    /// Tests to test the Basic Node Operations for the Network
    /// </summary>
    [TestClass]
    public class BasicNodeOperations
    {
        private static Container _ennfContainer;
        private static IUnityContainer _container;
        private static BasicFeedForwardPersistenceModelFixture _persistenceModel;

        [ClassInitialize]
        public static void MyClassInitialize(TestContext testContext)
        {
            _container = new UnityContainer();
            _container.LoadConfiguration(Settings.Default.UnityContainerName);

            _ennfContainer = new Container(_container);
            // It really doesn't matter what we try to load here, we just need to initialize
            // the network model and the rest will be taken care by the injected Model fixtre
            _ennfContainer.LoadFile(Resources.TestXmlFile);
            _persistenceModel = _ennfContainer.NetworkModel.PersistenceModel as BasicFeedForwardPersistenceModelFixture;
            Assert.IsNotNull(_persistenceModel);
        }

        [TestInitialize]
        public void MyTestInitialize()
        {
            _persistenceModel.ResetData();
        }


        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void AddNodeToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            INode newNode = network.CreateNode();
            network.Nodes.Add(newNode);
            _ennfContainer.NetworkModel.Save();

            Assert.AreEqual(_persistenceModel.NodeDbSet.AddCount, 1);
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void UpdateNodeToNetwork()
        {
            var rand = new Random();
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            INode node = network.Nodes.First();

            int resultNum = rand.Next(0, 1000);
            node.Address.AddressY = resultNum;
            _ennfContainer.NetworkModel.Save();
            // include a validation for the changes
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void RemoveNodeToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            INode node = network.Nodes.First();
            network.Nodes.Remove(node);

            _ennfContainer.NetworkModel.Save();

            Assert.AreEqual(_persistenceModel.NodeDbSet.RemoveCount, 1);
            Assert.AreEqual(_persistenceModel.ConnectionDbSet.RemoveCount, 3);
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void AddInputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInput input = new TestInput();

            network.Input = input;
            _ennfContainer.NetworkModel.Save();
            // check update
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void RemoveInputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);

            IInput input = network.Input;
            Assert.IsNotNull(input);
            network.Input = null;
            _ennfContainer.NetworkModel.Save();
            // check removal
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void UpdateInputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);

            IInput input = network.Input;
            Assert.IsNotNull(input);
            network.Input = new TestInputTwo();
            _ennfContainer.NetworkModel.Save();
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void ReadInputsOfNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IInput input = network.Input;
            Assert.IsNotNull(input);
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void AddOutputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutput output = new TestOutput();

            network.Output = output;
            _ennfContainer.NetworkModel.Save();
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void RemoveOutputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);

            IOutput output = network.Output;
            Assert.IsNotNull(output);
            network.Output = null;
            _ennfContainer.NetworkModel.Save();
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void UpdateOutputsToNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);

            IOutput output = network.Output;
            Assert.IsNotNull(output);
            network.Output = new TestOutputTwo();
            _ennfContainer.NetworkModel.Save();
        }

        [TestCategory("UnitTests"), TestCategory("Test Node Operations"), TestMethod]
        public void ReadOutputsOfNetwork()
        {
            INetwork network = _ennfContainer.NetworkModel.LoadNetwork(1);
            IOutput output = network.Output;
            Assert.IsNotNull(output);
        }

        [TestCleanup]
        public void FinalizeData()
        {
        }
    }
}